package com.ipsa.learn.controller;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.ipsa.learn.entity.*;
import com.ipsa.learn.service.CourseApplicationService;
import com.ipsa.learn.service.CourseExaminationService;
import com.ipsa.learn.service.CourseTransferService;
import com.ipsa.learn.service.CurriculumService;
import com.ipsa.learn.util.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 代课申请 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-31
 */
@Slf4j
@Api(tags = "课表展示 代课申请")
@RestController
@RequestMapping("/courseApplication")
public class CourseApplicationController {

    @Autowired
    CourseApplicationService courseApplicationService;

    @Autowired
    CurriculumService curriculumService;

    @Autowired
    CourseTransferService courseTransferService;

    @Autowired
    CourseExaminationService courseExaminationService;

    /**
     * 课表查询
     * @param scheduleDto
     * @return
     */
    @ApiOperation("标准课表展示")
    @PostMapping("/getTreeSchedule")
    public ResponseModel<List<CurriculumVo>> getTreeSchedule(@RequestBody ScheduleDto scheduleDto){
        log.info("**********courseAppcationDto*课表查询********,:{}",scheduleDto.toString());
        try {
            List<CurriculumVo> curriculumList = new ArrayList<>();

            if (scheduleDto != null){
                //标准排课表
                List<Curriculum> curriculum = curriculumService.getCurriculum(scheduleDto);
                //将String类型的字符串转换成LocalDateTime进行时间范围查询
                String stringStartTimeDto = scheduleDto.getStringStartTimeDto();
                String stringsLastTimeDto = scheduleDto.getStringsLastTimeDto();
                LocalDateTime startDate = LocalDateTime.parse(stringStartTimeDto+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                LocalDateTime lastDateTime = LocalDateTime.parse(stringsLastTimeDto+" 23:59:59", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                scheduleDto.setStartTimeDto(startDate);
                scheduleDto.setLastTimeDto(lastDateTime);
                //代课
                scheduleDto.setStatusDto(CheckStatus.PASS.getValue());
                List<CourseApplication> courseApplication = courseApplicationService.getCourseApplication(scheduleDto);
                //创建一个Map，将代课表中的数据存入Map中
                Map<String,CourseApplication> courseApplicationMap=new HashMap<>();
                for(CourseApplication CourseApplication:courseApplication){
                    courseApplicationMap.put(CourseApplication.getWeek()+CourseApplication.getClassPeriod(),CourseApplication);
                }
                //调课
                //List<CourseTransfer> courseTransfer = courseTransferService.getCourseTransfer(scheduleDto);
                List<CourseExamination> courseTransfer = courseExaminationService.getCourseExaminationList(scheduleDto);
                //创建一个Map，将调课表中的数据存入Map中
                Map<String,CourseExamination> courseTransferMap = new HashMap<>();
                for (CourseExamination transfer : courseTransfer) {
                    courseTransferMap.put(transfer.getWeek()+transfer.getClassPeriod(),transfer);
                }
                //遍历排课表中的数据，将存入map中的代课与调课数据放入排课表中，并存入list集合中
                for(Curriculum Curriculum:curriculum){
                    CurriculumVo curriculumVo=new CurriculumVo();
                    curriculumVo.setClassPeriod(Curriculum.getClassPeriod());
                    /**
                     * 周一课程   01代课   02调课   00正常
                     */
                     CurriculumNo mondayCurriculum01=new CurriculumNo();
                      if(null!=courseApplicationMap.get("周一"+Curriculum.getClassPeriod())){
                          mondayCurriculum01.setCourseCodeNo("01");
                          mondayCurriculum01.setCurriculumNameNo(courseApplicationMap.get("周一"+Curriculum.getClassPeriod()).getCurriculumName());  //代课的名称
                          mondayCurriculum01.setTeacherNameNo(courseApplicationMap.get("周一"+Curriculum.getClassPeriod()).getReplaceName());  //代课老师

                      }else if(null!=courseTransferMap.get("周一"+Curriculum.getClassPeriod())){
                        mondayCurriculum01.setCourseCodeNo("02");
                        mondayCurriculum01.setCurriculumNameNo(courseTransferMap.get("周一"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum01.setTeacherNameNo(courseTransferMap.get("周一"+Curriculum.getClassPeriod()).getReplaceName());
                    }else {
                          mondayCurriculum01.setCourseCodeNo("00");
                          mondayCurriculum01.setCurriculumNameNo(Curriculum.getMondayCurriculum());
                          mondayCurriculum01.setTeacherNameNo(Curriculum.getMondayTeacher());
                      }
                    curriculumVo.setMondayCurriculum(mondayCurriculum01);
                    /**
                     * 周二课程
                     */
                    CurriculumNo mondayCurriculum02=new CurriculumNo();
                    if(null!=courseApplicationMap.get("周二"+Curriculum.getClassPeriod())){
                        mondayCurriculum02.setCourseCodeNo("01");
                        mondayCurriculum02.setCurriculumNameNo(courseApplicationMap.get("周二"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum02.setTeacherNameNo(courseApplicationMap.get("周二"+Curriculum.getClassPeriod()).getReplaceName());
                    }else if(null!=courseTransferMap.get("周二"+Curriculum.getClassPeriod())){
                        mondayCurriculum02.setCourseCodeNo("02");
                        mondayCurriculum02.setCurriculumNameNo(courseTransferMap.get("周二"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum02.setTeacherNameNo(courseTransferMap.get("周二"+Curriculum.getClassPeriod()).getReplaceName());
                    }else {
                        mondayCurriculum02.setCourseCodeNo("00");
                        mondayCurriculum02.setCurriculumNameNo(Curriculum.getTuesdayCurriculum());
                        mondayCurriculum02.setTeacherNameNo(Curriculum.getTuesdayTeacher());
                    }
                    curriculumVo.setTuesdayCurriculum(mondayCurriculum02);
                    /**
                     * 周三课程
                     */
                    CurriculumNo mondayCurriculum03=new CurriculumNo();
                    if(null!=courseApplicationMap.get("周三"+Curriculum.getClassPeriod())){
                        mondayCurriculum03.setCourseCodeNo("01");
                        mondayCurriculum03.setCurriculumNameNo(courseApplicationMap.get("周三"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum03.setTeacherNameNo(courseApplicationMap.get("周三"+Curriculum.getClassPeriod()).getReplaceName());
                    }else if(null!=courseTransferMap.get("周三"+Curriculum.getClassPeriod())){
                        mondayCurriculum03.setCourseCodeNo("02");
                        mondayCurriculum03.setCurriculumNameNo(courseTransferMap.get("周三"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum03.setTeacherNameNo(courseTransferMap.get("周三"+Curriculum.getClassPeriod()).getReplaceName());
                    }else {
                        mondayCurriculum03.setCourseCodeNo("00");
                        mondayCurriculum03.setCurriculumNameNo(Curriculum.getWednesdayCurriculum());
                        mondayCurriculum03.setTeacherNameNo(Curriculum.getWednesdayTeacher());
                    }
                    curriculumVo.setWednesdayCurriculum(mondayCurriculum03);

                    /**
                     * 周四课程
                     */
                    CurriculumNo mondayCurriculum04=new CurriculumNo();
                    if(null!=courseApplicationMap.get("周四"+Curriculum.getClassPeriod())){
                        mondayCurriculum04.setCourseCodeNo("01");
                        mondayCurriculum04.setCurriculumNameNo(courseApplicationMap.get("周四"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum04.setTeacherNameNo(courseApplicationMap.get("周四"+Curriculum.getClassPeriod()).getReplaceName());
                    }else if(null!=courseTransferMap.get("周四"+Curriculum.getClassPeriod())){
                        mondayCurriculum04.setCourseCodeNo("02");
                        mondayCurriculum04.setCurriculumNameNo(courseTransferMap.get("周四"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum04.setTeacherNameNo(courseTransferMap.get("周四"+Curriculum.getClassPeriod()).getReplaceName());
                    }else {
                        mondayCurriculum04.setCourseCodeNo("00");
                        mondayCurriculum04.setCurriculumNameNo(Curriculum.getThursdayCurriculum());
                        mondayCurriculum04.setTeacherNameNo(Curriculum.getThursdayTeacher());
                    }
                    curriculumVo.setThursdayCurriculum(mondayCurriculum04);
                    /**
                     * 周五课程
                     */
                    CurriculumNo mondayCurriculum05=new CurriculumNo();
                    if(null!=courseApplicationMap.get("周五"+Curriculum.getClassPeriod())){
                        mondayCurriculum05.setCourseCodeNo("01");
                        mondayCurriculum05.setCurriculumNameNo(courseApplicationMap.get("周五"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum05.setTeacherNameNo(courseApplicationMap.get("周五"+Curriculum.getClassPeriod()).getReplaceName());
                    }else if(null!=courseTransferMap.get("周五"+Curriculum.getClassPeriod())){
                        mondayCurriculum05.setCourseCodeNo("02");
                        mondayCurriculum05.setCurriculumNameNo(courseTransferMap.get("周五"+Curriculum.getClassPeriod()).getCurriculumName());
                        mondayCurriculum05.setTeacherNameNo(courseTransferMap.get("周五"+Curriculum.getClassPeriod()).getReplaceName());
                    }else {
                        mondayCurriculum05.setCourseCodeNo("00");
                        mondayCurriculum05.setCurriculumNameNo(Curriculum.getFridayCurriculum());
                        mondayCurriculum05.setTeacherNameNo(Curriculum.getFridayTeacher());
                    }
                    curriculumVo.setFridayCurriculum(mondayCurriculum05);

                    curriculumList.add(curriculumVo);
                }

                return ResponseModel.success(curriculumList).code(ResultCode.SUCCESS);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("课表查询错误").code(ResultCode.ERROR);
        }
        return null;
    }

    @ApiOperation("代课查询")
    @PostMapping("/getCourseApplication")
    public ResponseModel<PageResult> getCourseApplication(@RequestBody ScheduleDto scheduleDto){
        log.info("**********courseApplication代课查询***********, :{}",scheduleDto.toString());
        try {
            if (null == scheduleDto.getPageNum()){
                scheduleDto.setPageNum(PageParams.DEFAULT_PAGE_NUM);
            }
            if (null == scheduleDto.getPageSize()){
                scheduleDto.setPageSize(PageParams.DEFAULT_PAGE_SIZE);
            }
            PageHelper.startPage(scheduleDto.getPageNum(),scheduleDto.getPageSize());
            Page<CourseApplication> courseApplication = (Page<CourseApplication>)courseApplicationService.getCourseApplication(scheduleDto);
            return ResponseModel.success(new PageResult(courseApplication.getTotal(),courseApplication.getResult())).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("代课查询失败").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("代课审核")
    @PostMapping("/updateCourseApplication")
    public ResponseModel<String> updateCourseApplication(@RequestBody CourseApplication courseApplication){
        log.info("***********, :{}",courseApplication.getCurriculumId());

        try {
            if (courseApplication.getCurriculumId() != null){
                courseApplicationService.updateById(courseApplication);
                return ResponseModel.success("代课修改成功").code(ResultCode.SUCCESS);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("代课修改失败").code(ResultCode.ERROR);
        }
        return null;
    }

    @ApiOperation("代课申请")
    @PostMapping("/saveCourseApplication")
    public ResponseModel<String> saveCourseApplication(@RequestBody CurriculumDTO curriculumDTO){
        log.info("******代课申请curriculumDTO*****, :{}",curriculumDTO.toString());
        try {
            if (curriculumDTO != null){
                Boolean checkCourseApplicationTransfer = getCheckCourseApplicationTransfer(curriculumDTO);
                if (checkCourseApplicationTransfer){
                    return ResponseModel.success("申请重复，请重新选择。").code(ResultCode.SUCCESS);
                }
                //将String类型的字符串转换成LocalDateTime进行时间范围查询
                String stringClassTimeDto = curriculumDTO.getStringClassTimeDto();
                LocalDateTime localDateTime = LocalDateTime.parse(stringClassTimeDto+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
               curriculumDTO.setClassTimeDto(localDateTime);
                CourseApplication courseApplication = getCourseApplications(curriculumDTO);
                courseApplicationService.save(courseApplication); //代課申請保存
                return ResponseModel.success("代课申请保存成功").code(ResultCode.SUCCESS);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("*******代课申请保存失败*******, :{}",e.getMessage());
            return ResponseModel.failure("代课申请保存失败").code(ResultCode.ERROR);
        }
        return null;
    }


    private CourseApplication getCourseApplications(CurriculumDTO curriculumDTO){
        CourseApplication courseApplication = new CourseApplication();
        courseApplication.setSchoolCode(curriculumDTO.getSchoolCode()); //学校编码
        courseApplication.setSchoolYear(curriculumDTO.getSchoolYear());  //学年
        courseApplication.setSemester(curriculumDTO.getSemester());  //学期
        courseApplication.setGrade(curriculumDTO.getGrade());  //年级
        courseApplication.setGradeName(curriculumDTO.getGradeName()); //年级名称
        courseApplication.setClassCode(curriculumDTO.getClassCode());  //班级
        courseApplication.setClassName(curriculumDTO.getClassName()); //班级名称
        courseApplication.setClassPeriod(curriculumDTO.getClassPeriod());  //课节
        courseApplication.setApplicationTime(curriculumDTO.getClassTimeDto());//代课的时间
        courseApplication.setWeek(curriculumDTO.getWeekDto()); //星期几
        courseApplication.setCurriculumName(curriculumDTO.getCurriculumNameDto());//课程名称
        courseApplication.setApplicationName(curriculumDTO.getApplicationNameDto());//申请人
        courseApplication.setReplaceName(curriculumDTO.getReplaceNameDto());//代课人
        courseApplication.setCreateUser(curriculumDTO.getCreateUser());//创建人
        //courseApplication.setCreateTime(DateUtil.DateToLocalDateTime());//创建时间
        courseApplication.setYn(ChooseStatus.NORMAL.getValue()); //正常
        courseApplication.setStatus(CheckStatus.DURING.getValue()); //待申核
        courseApplication.setReasons(curriculumDTO.getReasonsDto()); //申請原因
        courseApplication.setRemarks(curriculumDTO.getRemarksDto()); //說明
        return courseApplication;
    }

    private Boolean getCheckCourseApplicationTransfer(CurriculumDTO curriculumDTO){
        List<CourseTransfer> checkCourseTransfer = courseTransferService.getCheckCourseTransfer(curriculumDTO);
        List<CourseApplication> checkCourseApplication = courseApplicationService.getCheckCourseApplication(curriculumDTO);
        if (checkCourseTransfer != null && checkCourseTransfer.size()>0 ||checkCourseApplication != null && checkCourseApplication.size()>0){
            return true;
        }
        return false;
    }

}