package com.ipsa.learn.controller;


import com.ipsa.learn.entity.*;
import com.ipsa.learn.service.CourseDailycheckService;
import com.ipsa.learn.util.ResponseModel;
import com.ipsa.learn.util.ResultCode;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * <p>
 * 日常检查 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-09-03
 */
@Slf4j
@Api(tags = "日常检查")
@RestController
@RequestMapping("/courseDailycheck")
public class CourseDailycheckController {

    @Autowired
    CourseDailycheckService courseDailycheckService;

    @ApiOperation("日常检查添加")
    @PostMapping("/save")
    public ResponseModel<String> saveDailycheck(@RequestBody CourseDailycheck courseDailycheck){
        log.info("*******courseDailycheck****,:{}",courseDailycheck.toString());
        try {
            courseDailycheckService.save(courseDailycheck);
            return ResponseModel.success("日常检查添加成功").code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("日常检查添加失败").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("日常检查查询按时间")
    @PostMapping("/getgetDailycheckList")
    public ResponseModel<List<CourseDailycheck>> getDailycheckList(@RequestBody ScheduleDto scheduleDto){
        log.info("*********scheduleDto*******, :{}",scheduleDto);
        try {
            //将String类型的字符串转换成LocalDateTime进行时间范围查询
            String stringStartTimeDto = scheduleDto.getStringStartTimeDto();
            String stringsLastTimeDto = scheduleDto.getStringsLastTimeDto();
            LocalDateTime startDate = LocalDateTime.parse(stringStartTimeDto+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            LocalDateTime lastDateTime = LocalDateTime.parse(stringsLastTimeDto+" 23:59:59", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            scheduleDto.setStartTimeDto(startDate);
            scheduleDto.setLastTimeDto(lastDateTime);
            List<CourseDailycheck> dailycheckList = courseDailycheckService.getDailycheckList(scheduleDto);
            return ResponseModel.success(dailycheckList).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询日常检查失败").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("按学校和年级检查查询流动红旗")
    @PostMapping("/getRedFlag")
    public ResponseModel<RedFlagResponse> getRedFlag(@RequestBody RedFlagRequest request) {
        log.info("*********RedFlagRequest*******, :{}", request);
        try {

            RedFlagResponse response = new RedFlagResponse();
            List<RedFlag> list = courseDailycheckService.getRedFlag(request);
            response.setRedFlags(list);

//            List<RedFlagScore> score = courseDailycheckService.getRedFlagScore(request);
//            response.setScores(score);


            //获取上周排榜
            request.setStart(request.getLastStart());
            request.setEnd(request.getLastEnd());
            List<RedFlag> last = courseDailycheckService.getRedFlag(request);

            for (RedFlag redFlag : list) {
                setRedFlagLastRank(redFlag, last);
            }


            return ResponseModel.success(response).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询流动红旗失败").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("按学校和年级和班级检查查询流动红旗雷达图")
    @PostMapping("/getRedFlagChart")
    public ResponseModel<List<RedFlagScore>> getRedFlagChart(@RequestBody RedFlagRequest request) {
        log.info("*********RedFlagRequest*******, :{}", request);
        try {


            List<RedFlagScore> score = courseDailycheckService.getRedFlagScore(request);

            return ResponseModel.success(score).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询流动红旗失败").code(ResultCode.ERROR);
        }
    }

    private void setRedFlagLastRank(RedFlag redFlag, List<RedFlag> last) {
        for (RedFlag item : last) {
            if (item.getGradeName().equals(redFlag.getGradeName())
                    && item.getClassName().equals(redFlag.getClassName())) {
                redFlag.setLastRank(item.getRank());
            }
        }
    }

}

