package com.ipsa.learn.controller;


import com.alibaba.fastjson.JSONObject;
import com.ipsa.learn.entity.*;
import com.ipsa.learn.entity.dto.UserInfoDto;
import com.ipsa.learn.service.CourseResultlistService;
import com.ipsa.learn.service.CourseResultsuploadService;
import com.ipsa.learn.util.DateUtil;
import com.ipsa.learn.util.ResponseModel;
import com.ipsa.learn.util.ResultCode;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 成绩列表 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-09-04
 */
@Slf4j
@RestController
@Api(tags = "成绩列表")
@RequestMapping("/courseResultlist")
public class CourseResultlistController {

    @Autowired
    CourseResultlistService courseResultlistService;

    @Autowired
    CourseResultsuploadService courseResultsuploadService;

    @ApiOperation("查询学生成绩排名")
    @PostMapping("/getScoreSortingList")
    public ResponseModel<List<StudentTotalScore>> getScoreSortingList(@RequestBody RankByClass rankByClass) {
        log.info("CourseResultlistController.getScoreSortingList params:{}", rankByClass);
        try {
            List<StudentTotalScore> scoreSortingList = courseResultlistService.getScoreSortingList(rankByClass);
            return ResponseModel.success(scoreSortingList).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            log.error("CourseResultlistController.getScoreSortingList erro", e);
            return ResponseModel.failure("查询信息有误").code(ResultCode.ERROR);
        }
    }
    @ApiOperation("查询学生成绩详情")
    @PostMapping("/getStudentScoreDetails")
    public ResponseModel<StudentScoreDetails> getStudentScoreDetails(@RequestBody StudentScoreDto studentScoreDto) {
        log.info("CourseResultlistController.getStudentScoreDetails params:{}", "");
        try {
            StudentScoreDetails studentScoreDetails = courseResultlistService.getStudentScoreDetails(studentScoreDto);
            return ResponseModel.success(studentScoreDetails).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            log.error("CourseResultlistController.getStudentScoreDetails erro", e);
            return ResponseModel.failure("查询学生成绩详情有误").code(ResultCode.ERROR);
        }

    }
    @ApiOperation("获取学生考试科目列表")
    @PostMapping("/getsubjectsName")
    public ResponseModel<List<Map<String,String>>> getsubjectsName(@RequestBody StudentScoreDto studentScoreDto) {
        List<Map<String, String>> subjectsName = courseResultlistService.getSubjectsName(studentScoreDto);
        return ResponseModel.success(subjectsName).code(ResultCode.SUCCESS);
    }
    @ApiOperation("家长查询孩子成绩列表")
    @PostMapping("/getParentsList")

    public ResponseModel<List<CourseSoreVo>> getParentsList(@RequestBody CourseResultsuploadVo courseResultsupload) {
        try {
            log.info("******courseResultsupload*******, :{}", courseResultsupload.toString());
            List<CourseSoreVo> parentsList = courseResultlistService.getParentsList(courseResultsupload);
            return ResponseModel.success(parentsList).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询信息有误").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("成绩列表查询")
    @PostMapping("/getResultList")
    public ResponseModel<List<CourseResultlist>> getResultList(@RequestBody CourseResultsupload courseResultsupload) {
        try {
            List<CourseResultlist> resultList = courseResultlistService.getResultList(courseResultsupload);
            return ResponseModel.success(resultList).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询信息有误").code(ResultCode.ERROR);
        }
    }


    @ApiOperation("得到评价的下拉列表")
    @GetMapping("/getCommentTypeList")
    public ResponseModel getCommentTypeList() {
        // todo 以下列表会根据具体业务进行修改
        String[] typeList = {"评语1", "评语2", "评语3", "自定义"};
        return ResponseModel.success(typeList).code(ResultCode.SUCCESS);
    }

    @ApiOperation("对学员进行评价")
    @PostMapping("/commentByStudent")
    public ResponseModel commentByStudent(@RequestBody CommentByStudent dto, HttpServletRequest httpServletRequest) {
        log.info("*****CommentByStudentDto**** :{}", dto);
//        String userInfo = httpServletRequest.getHeader("userInfo");
//        JSONObject jsonObject = JSONObject.parseObject(userInfo);
//        UserInfoDto userInfoDto = JSONObject.toJavaObject(jsonObject, UserInfoDto.class);
        int i = courseResultlistService.addCommentByStudent(dto);
        if (i > 0) {
            return ResponseModel.success(i).code(ResultCode.SUCCESS);
        }
        return ResponseModel.failure(null).code(ResultCode.ERROR);
    }

    @ApiOperation("获取老师对学生的评价")
    @GetMapping("/getCommentByStudent/{studentId}")
    public ResponseModel getCommentByStudent(@PathVariable String studentId) {
        log.info("*****CommentByStudentDto**** :{}", studentId);

        List<CommentByStudent> commentByStudent = courseResultlistService.getCommentByStudent(studentId);
        return ResponseModel.success(commentByStudent).code(ResultCode.SUCCESS);
    }

    @ApiOperation("成绩上传所填数据的查询")
    @GetMapping("/getGradeuploading/{courseUploadId}")
    public ResponseModel<CourseResultsupload> getGradeuploading(@PathVariable("courseUploadId") String courseUploadId) {
        try {
            log.info("*****courseUploadId**** :{}", courseUploadId);
            CourseResultsupload byId = courseResultsuploadService.getById(courseUploadId);
            Date date = DateUtil.localDateTime2Date(byId.getTestdateTime());
            String date1 = DateUtil.dateToString(date);
            byId.setStringTestdateTime(date1);
            System.out.println(date1);
            return ResponseModel.success(byId).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("成绩上传数据查询错误").code(ResultCode.ERROR);
        }
    }

}

