package com.ipsa.learn.controller;


import com.dingtalk.api.response.OapiEduStudentListResponse;
import com.ipsa.learn.entity.*;
import com.ipsa.learn.service.CourseScoreService;
import com.ipsa.learn.util.ResponseModel;
import com.ipsa.learn.util.ResultCode;
import com.ipsa.learn.util.UUIDGenerator;
import com.ipsa.learn.util.dingtalkUtil.CssDingUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 科目分数表 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-09-03
 */
@Slf4j
@RestController
@Api(tags = "科目分数表")
@RequestMapping("/courseScore")
public class CourseScoreController {

    @Autowired
    CourseScoreService courseScoreService;


    @ApiOperation("上传导入Excel中的成绩表信息")
    @PostMapping("/uploads")
    public ResponseModel<String> uplodeExcels(MultipartFile file,CourseResultsuploadVo courseResultsupload){
        log.info("******courseResultsupload 成绩上传相关的信息******, :{}",courseResultsupload.toString());
        try {
            String uploadName = courseScoreService.uploadExcels(file, courseResultsupload);
            return ResponseModel.success(uploadName).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("导出Excel中的课程信息存在错误").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("上传成绩导出Excel模板")
    @GetMapping("/reports/{classId}/{sub_name}")
    public void report(HttpServletRequest request, HttpServletResponse response, @PathVariable("classId") String classId, @PathVariable("sub_name") String sub_name) throws IOException {

        //String sub_name = request.getParameter("sub_name");
        long l = Long.parseLong(classId);
        long pageSize = 10;
        List<List<OapiEduStudentListResponse.StudentRespone>> objects = new ArrayList<>();
        for (long i = 1; i <= 1000; i++) {
            OapiEduStudentListResponse studentList = CssDingUtil.getStudentList(l, i, pageSize);
            OapiEduStudentListResponse.PageResult result = studentList.getResult();
            if (null != result){
                List<OapiEduStudentListResponse.StudentRespone> list = result.getList();
                objects.add(list);
            }else {
                break;
            }
        }
        List<CourseScore> list = new ArrayList<>();
        for (List<OapiEduStudentListResponse.StudentRespone> object : objects) {
            for (OapiEduStudentListResponse.StudentRespone studentRespone : object) {
                CourseScore courseScore = new CourseScore();
                String name = studentRespone.getName(); //学生姓名
                String studentNo = studentRespone.getStudentNo();  //学生编号
                String studentUserid = studentRespone.getStudentUserid();  //学生ID
                courseScore.setStudentUserId(studentUserid);
                courseScore.setStudentNo(studentNo);
                courseScore.setStudentUserName(name);
                list.add(courseScore);
            }
        }
        String[] curriculum_Name = new String[30];
        curriculum_Name[0] = "学生Id";
        curriculum_Name[1] = "学生编号";
        curriculum_Name[2] = "学生姓名";
        //{"学生Id","学生编号","学生名称",sub_name+"成绩"};
        String[] split = sub_name.split(",");
        List<String> subNameList = new ArrayList<>(Arrays.asList(split));
        for (int i = 0; i < subNameList.size(); i++) {
            curriculum_Name[i + 3] = subNameList.get(i) + "成绩";
        }

        HSSFWorkbook wb = new HSSFWorkbook();
        HSSFSheet sheet = wb.createSheet("成绩上传模板");
        HSSFRow row1 = sheet.createRow(0);
        for (int i = 0; i < curriculum_Name.length; i++) {
            row1.createCell(i).setCellValue(curriculum_Name[i]);
        }

        for (int i = 0; i < list.size(); i++) {
            HSSFRow row = sheet.createRow(i + 1);
            row.createCell(0).setCellValue(list.get(i).getStudentUserId());
            row.createCell(1).setCellValue(list.get(i).getStudentNo());
            row.createCell(2).setCellValue(list.get(i).getStudentUserName());
        }
        OutputStream outputStream = response.getOutputStream();
        response.reset();
        response.setHeader("Content-disposition", "attachment; filename="+ UUIDGenerator.getUUID()+".xls");
        response.setContentType("application/msexcel");
        wb.write(outputStream);
        outputStream.close();

    }


    @ApiOperation("历史成绩查询")
    @PostMapping("/history")
    public ResponseModel<List<CourseScoreHistory>> getHistory(@RequestBody CourseScoreHistoryRequest request) {
        log.info("历史成绩查询,请求参数={}",request);
        try{
            List<CourseScoreHistory> list=courseScoreService.getHistory(request);
            return ResponseModel.success(list).code(ResultCode.SUCCESS);
        }catch (Exception ex){
            ex.printStackTrace();
            log.info("历史成绩查询失败:{}",ex);
            return ResponseModel.failure("历史成绩查询失败").code(ResultCode.SUCCESS);
        }
    }

}

