package com.ipsa.learn.controller;


import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.ipsa.learn.entity.CourseApplication;
import com.ipsa.learn.entity.CourseTransfer;
import com.ipsa.learn.entity.CurriculumDTO;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.CourseApplicationService;
import com.ipsa.learn.service.CourseExaminationService;
import com.ipsa.learn.service.CourseTransferService;
import com.ipsa.learn.util.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * <p>
 * 调课申请 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-31
 */
@Slf4j
@Api(tags = "课表展示 调课申请")
@RestController
@RequestMapping("/courseTransfer")
public class CourseTransferController {

    @Autowired
    CourseTransferService courseTransferService;

    @Autowired
    CourseExaminationService courseExaminationService;

    @Autowired
    CourseApplicationService courseApplicationService;

    @ApiOperation("查询调课")
    @PostMapping("/getCourseTransfer")
    public ResponseModel<PageResult> getCourseTransfer(@RequestBody ScheduleDto scheduleDto){
        log.info("*********查询课程接口: courseTransferDto*********，:{}",scheduleDto.toString());
        try {
            if (scheduleDto.getPageNum() == null){
                scheduleDto.setPageNum(PageParams.DEFAULT_PAGE_NUM);
            }
            if (scheduleDto.getPageSize() == null){
                scheduleDto.setPageSize(PageParams.DEFAULT_PAGE_SIZE);
            }
            PageHelper.startPage(scheduleDto.getPageNum(),scheduleDto.getPageSize());
            Page<CourseTransfer> courseTransfer = (Page<CourseTransfer>)courseTransferService.getCourseTransfer(scheduleDto);
            List<CourseTransfer> result = courseTransfer.getResult();
            return ResponseModel.success(new PageResult(courseTransfer.getTotal(),result)).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询调课信息失败").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("调课审核")
    @PostMapping("/updateCourseTransfer")
    public ResponseModel<String> updateCourseTransfer(@RequestBody CourseTransfer courseTransfer){
        log.info("******调课审核*******,{}",courseTransfer.toString());
        try {
            if (courseTransfer != null){
                courseTransferService.updateById(courseTransfer);
                if (courseTransfer.getStatus() == CheckStatus.PASS.getValue()){
                    courseExaminationService.saveCourseExamination(courseTransfer.getCourseTransferId());
                }
            }
            return ResponseModel.success("调课审核成功").code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("调课修改失败").code(ResultCode.ERROR);
        }
    }

    @ApiOperation("调课申请")
    @PostMapping("/saveCourseTransfer")
    public ResponseModel<String> saveCourseTransfer(@RequestBody CurriculumDTO curriculumDTO){
        log.info("******代课申请curriculumDTO*****, :{}",curriculumDTO.toString());
        try {
            if (curriculumDTO != null){
                Boolean checkCourseApplicationTransfer = getCheckCourseApplicationTransfer(curriculumDTO);
                if (checkCourseApplicationTransfer){
                    return ResponseModel.success("申请重复，请重新选择。").code(ResultCode.SUCCESS);
                }
                String stringClassTimeDto = curriculumDTO.getStringClassTimeDto();//调课的时间
                String stringApplicationTimeGradeDto = curriculumDTO.getStringApplicationTimeGradeDto();//被调课的时间
                LocalDateTime localDateTime = LocalDateTime.parse(stringClassTimeDto+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                LocalDateTime localDateTime1 = LocalDateTime.parse(stringApplicationTimeGradeDto+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                curriculumDTO.setClassTimeDto(localDateTime);
                curriculumDTO.setApplicationTimeGradeDto(localDateTime1);
                CourseTransfer courseTransfer = getCourseTransfers(curriculumDTO);
                courseTransferService.save(courseTransfer); //代課申請保存
                return ResponseModel.success("代课申请保存成功").code(ResultCode.SUCCESS);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.info("*******代课申请保存失败*******, :{}",e.getMessage());
            return ResponseModel.failure("代课申请保存失败").code(ResultCode.ERROR);
        }
        return null;
    }

    private CourseTransfer getCourseTransfers(CurriculumDTO curriculumDTO){
        CourseTransfer  courseTransfer = new CourseTransfer();
        courseTransfer.setSchoolCode(curriculumDTO.getSchoolCode()); //学校编码
        courseTransfer.setSchoolYear(curriculumDTO.getSchoolYear());  //学年
        courseTransfer.setSemester(curriculumDTO.getSemester());  //学期
        courseTransfer.setGrade(curriculumDTO.getGrade());  //年级
        courseTransfer.setGradeName(curriculumDTO.getGradeName());  //年级名称
        courseTransfer.setClassCode(curriculumDTO.getClassCode());  //班级
        courseTransfer.setClassName(curriculumDTO.getClassName());  //班级名称
        courseTransfer.setClassPeriod(curriculumDTO.getClassPeriod());  //课节
        courseTransfer.setApplicationTime(curriculumDTO.getClassTimeDto());//调课的时间
        courseTransfer.setWeek(curriculumDTO.getWeekDto()); //星期几
        courseTransfer.setCurriculumName(curriculumDTO.getCurriculumNameDto());//课程名称
        courseTransfer.setApplicationName(curriculumDTO.getApplicationNameDto());//申请人
        courseTransfer.setReplaceName(curriculumDTO.getReplaceNameDto());//代课人
        courseTransfer.setCreateUser(curriculumDTO.getCreateUser());//创建人
        //courseTransfer.setCreateTime(DateUtil.DateToLocalDateTime());//创建时间
        courseTransfer.setYn(ChooseStatus.NORMAL.getValue()); //正常
        courseTransfer.setGradeGrade(curriculumDTO.getGradeDto()); //调年级
        courseTransfer.setGradeGradeName(curriculumDTO.getGradeGradeNameDto());  //调年级名称
        courseTransfer.setClassGrade(curriculumDTO.getClassGradeDto()); //调班级
        courseTransfer.setClassGradeName(curriculumDTO.getClassGradeNameDto());  //调班级名称
        courseTransfer.setClassPeriodGrade(curriculumDTO.getClassPeriodGradeDto()); //调课节
        courseTransfer.setApplicationTimeGrade(curriculumDTO.getApplicationTimeGradeDto());//被调课时间
        courseTransfer.setWeekGrade(curriculumDTO.getWeekGradeDto());//调星期几
        courseTransfer.setCurriculumNameGrade(curriculumDTO.getCurriculumNameGradeDto());//调课程名称
        courseTransfer.setStatus(CheckStatus.DURING.getValue()); //待申核
        courseTransfer.setReasons(curriculumDTO.getReasonsDto()); //申請原因
        courseTransfer.setRemarks(curriculumDTO.getRemarksDto()); //說明
        return courseTransfer;
    }

    private Boolean getCheckCourseApplicationTransfer(CurriculumDTO curriculumDTO){
        List<CourseTransfer> checkCourseTransfer = courseTransferService.getCheckCourseTransfer(curriculumDTO);
        List<CourseApplication> checkCourseApplication = courseApplicationService.getCheckCourseApplication(curriculumDTO);
        if (checkCourseTransfer != null && checkCourseTransfer.size()>0 ||checkCourseApplication != null && checkCourseApplication.size()>0){
            return true;
        }
        return false;
    }


}

