package com.ipsa.learn.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ipsa.learn.entity.Curriculum;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.CurriculumService;
import com.ipsa.learn.util.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

/**
 * <p>
 * 排课表 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-24
 */
@Slf4j
@Api(tags = "排课表")
@RequestMapping("/curriculum")
@RestController
public class CurriculumController {

    @Autowired
    CurriculumService curriculumService;

    @ApiOperation("导出Excel模板")
    @GetMapping("/report")
    public void report(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String[] curriculum_Name = new String[]{"周一科目","周一老师","周二科目","周二老师","周三科目","周三老师","周四科目","周四老师","周五科目","周五老师"};
        HSSFWorkbook wb = new HSSFWorkbook();
        HSSFSheet sheet = wb.createSheet("录入课表");
        HSSFRow row1 = sheet.createRow(0);
        for (int i = 0; i < curriculum_Name.length; i++) {
            row1.createCell(i+1).setCellValue(curriculum_Name[i]);
        }
        String[] rights_Name = new String[]{"第一节","第二节","第三节","第四节","第五节","第六节","第七节","第八节"};
        for (int i = 0; i < rights_Name.length; i++) {
            HSSFRow row = sheet.createRow(i + 1);
            row.createCell(0).setCellValue(rights_Name[i]);
        }
        OutputStream outputStream = response.getOutputStream();
        response.reset();
        response.setHeader("Content-disposition", "attachment; filename="+ UUIDGenerator.getUUID()+".xls");
        response.setContentType("application/msexcel");
        wb.write(outputStream);
        outputStream.close();

    }

    @ApiOperation("导入Excel中的课表信息")
    @PostMapping("/upload")
    public ResponseModel<String> uplodeExcel(MultipartFile file,ScheduleDto scheduleDto){
        ResponseModel<String> responseModel = null;
        try {
            responseModel = curriculumService.uploadExcel(file, scheduleDto);
            return responseModel;
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("导出Excel中的课程信息存在错误").code(ResultCode.ERROR);
        }
    }

    //添加标准课排课
    @ApiOperation("添加标准课排课")
    @PostMapping("/save")
    public ResponseModel<String> save(@RequestBody List<Curriculum> curriculum){
        log.info("****curriculum****,:{}",curriculum.toString());
        try {
            LambdaQueryWrapper<Curriculum> queryWrapper = null;
            for (Curriculum curriculum1 : curriculum) {
                queryWrapper = new LambdaQueryWrapper<>();
                if (StringUtils.isNotEmpty(curriculum1.getSchoolCode())){
                    //学校编码
                    queryWrapper.eq(Curriculum::getSchoolCode,curriculum1.getSchoolCode());
                }
                if (StringUtils.isNotEmpty(curriculum1.getSchoolYear())){
                    //学年
                    queryWrapper.eq(Curriculum::getSchoolYear,curriculum1.getSchoolYear());
                }
                if (StringUtils.isNotEmpty(curriculum1.getSemester())){
                    //学期
                    queryWrapper.eq(Curriculum::getSemester,curriculum1.getSemester());
                }
                if (StringUtils.isNotEmpty(curriculum1.getGrade())){
                    //年级
                    queryWrapper.eq(Curriculum::getGrade,curriculum1.getGrade());
                }
                if (StringUtils.isNotEmpty(curriculum1.getClassCode())){
                    //班级
                    queryWrapper.eq(Curriculum::getClassCode,curriculum1.getClassCode());
                }
                String classPeriod = curriculum1.getClassPeriod(); //获取节课
                log.info("******classPeriod******, {}",classPeriod);
                queryWrapper.eq(Curriculum::getClassPeriod,curriculum1.getClassPeriod());
                Curriculum one = curriculumService.getOne(queryWrapper);
                if (one != null){
                    curriculum1.setUpdateTime(DateUtil.DateToLocalDateTime()); //修改的时间
                    curriculumService.updateById(curriculum1);
                }else {
                    curriculum1.setCreateTime(DateUtil.DateToLocalDateTime());  //创建的时间
                    curriculumService.save(curriculum1);
                }
            }
            return ResponseModel.success("添加标准排课列表成功").code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("添加标准排课列表失败").code(ResultCode.ERROR);
        }
    }

    /**
     * 排课表查询
     * @param scheduleDto
     * @return
     */
    @PostMapping("/tree")
    @ApiOperation("排课表查询")
    public ResponseModel<List<Curriculum>> getCurriculum(@RequestBody ScheduleDto scheduleDto){
        try {
            log.info("****curriculum****,:{}",scheduleDto.toString());
            List<Curriculum> curriculum1 = curriculumService.getCurriculum(scheduleDto);
            return ResponseModel.success(curriculum1).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            log.info("****查询排课异常curriculum****, :{}",scheduleDto.toString());
            e.printStackTrace();
            return ResponseModel.failure("查询排课异常").code(ResultCode.ERROR);
        }
    }

    /**
     * 根据排课表查询科目名称
     * @param scheduleDto
     * @return
     */
    @PostMapping("/getSubjectQuery")
    @ApiOperation("根据排课表查询科目名称")
    public ResponseModel<List<String>> getSubjectQuery(@RequestBody ScheduleDto scheduleDto){
        try {
            log.info("*******scheduleDto******* :{}",scheduleDto.toString());
            List<String> subjectQuery = curriculumService.getSubjectQuery(scheduleDto);
            return ResponseModel.success(subjectQuery).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("查询科目失败").code(ResultCode.ERROR);
        }
    }

}

