package com.ipsa.learn.controller;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ipsa.learn.entity.ElectiveCourse;
import com.ipsa.learn.entity.ElectiveCourseDto;
import com.ipsa.learn.entity.ElectiveEnroll;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.ElectiveCourseService;
import com.ipsa.learn.service.ElectiveEnrollService;
import com.ipsa.learn.util.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 选修课报名 前端控制器
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-24
 */
@Slf4j
@Api(tags = "选修课报名")
@RestController
@RequestMapping("/electiveEnroll")
public class ElectiveEnrollController {


    @Autowired
    ElectiveCourseService electiveCourseService;

    @Autowired
    ElectiveEnrollService electiveEnrollService;

    /**
     * 学生或老师查询选课
     * @return
     */
    @ApiOperation("学生或老师查询选修课")
    @PostMapping("/getcoursell")
    public ResponseModel<List<ElectiveCourseDto>> getCourseEnrollList(@RequestBody ScheduleDto electiveEnroll){
        try {
            //里面封装了学生学号，学校编号
            //ElectiveEnroll electiveEnroll = new ElectiveEnroll();
            List<ElectiveCourseDto> courseEnrollList = electiveCourseService.getCourseEnrollList(electiveEnroll);
            return ResponseModel.success(courseEnrollList).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 学生添加选课
     * @param electiveCourses
     * @return
     */
    @ApiOperation("学生添加选课")
    @PostMapping("/inertcourse")
    public ResponseModel<String> insertEnrollList(@RequestBody List<ElectiveCourseDto> electiveCourses){
        log.info("************添加学生选课信息*************");
        try {
            //获取当前登录人信息以及学校信息
            ElectiveEnroll electiveEnroll = null;
            for (ElectiveCourseDto electiveCours : electiveCourses) {
                log.info("****electiveCours :{}****",electiveCours.toString());
                //取消选修课，如果前端传递过来的选修课Id存在时，就相当于取消选修课
                if (StringUtils.isNotEmpty(electiveCours.getElectiveEnrollId())){
                    ElectiveEnroll electiveEnroll1 = electiveEnrollService.getById(electiveCours.getElectiveEnrollId());
                    //取消选修课
                    electiveEnroll1.setYn(0);
                    electiveEnrollService.updateById(electiveEnroll1);
                    Long electiveCourseId = electiveCours.getElectiveCourseId(); //获取添加排课Id
                    ElectiveCourse course = electiveCourseService.getById(electiveCourseId);
                    Integer enrollNumber = course.getEnrollNumber()-1 < 0 ? 0 : course.getEnrollNumber()-1;
                    course.setEnrollNumber(enrollNumber);
                    electiveCourseService.updateById(course);
                    return ResponseModel.success("取消选修课报名成功").code(ResultCode.SUCCESS);
                }else {
                    electiveEnroll  = new ElectiveEnroll();
                    electiveEnroll.setSchoolCode(electiveCours.getSchoolCode()); //学校编号
                    electiveEnroll.setCourseId(electiveCours.getElectiveCourseId().toString()); //选修课Id
                    electiveEnroll.setClassCode(electiveCours.getClassDto()); //班级  钉钉获取
                    electiveEnroll.setClassName(electiveCours.getClassNameDto()); //班级名称
                    electiveEnroll.setGrade(electiveCours.getGradeDto());  //年级
                    electiveEnroll.setGradeName(electiveCours.getGradeNameDto()); //年级名称
                    electiveEnroll.setStudentNumbe(electiveCours.getStudentNumbeDto());//学生学号  钉钉获取
                    electiveEnroll.setStudentName(electiveCours.getStudentNameDto());//学生名称   钉钉获取
                    electiveEnroll.setCreateUser(electiveCours.getCreateUser()); //创建人     钉钉获取
                    electiveEnroll.setStudentId(electiveCours.getStudentId());  //学生UserId
                    //  electiveEnroll.setUpdateUser(); //修改人     钉钉获取
                    electiveEnroll.setYn(ChooseStatus.NORMAL.getValue());
                    electiveEnrollService.save(electiveEnroll);
                    Long electiveCourseId = electiveCours.getElectiveCourseId();
                    ElectiveCourse course = electiveCourseService.getById(electiveCourseId);
                    if (null == course.getEnrollNumber()){
                        course.setEnrollNumber(0);
                    }
                    Integer enrollNumber = course.getEnrollNumber()+1;
                    log.info("******enrollNumber***** :{}",enrollNumber);
                    course.setEnrollNumber(enrollNumber);
                    electiveCourseService.updateById(course);
                    return ResponseModel.success("添加选修课报名成功").code(ResultCode.SUCCESS);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("添加选修课报名失败").code(ResultCode.ERROR);
        }
        return null;
    }

    /**
     * 查询报名列表
     * @param electiveCourse
     * @return
     */
    @ApiOperation("查询报名列表")
    @PostMapping("/getEnrollList")
    public ResponseModel< IPage<ElectiveEnroll> > getEnrollList(@RequestBody ElectiveCourse electiveCourse){
        log.info("****schoolCode**** :{}",electiveCourse.toString());

        try {
            LambdaQueryWrapper<ElectiveEnroll> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(ElectiveEnroll::getCourseId,electiveCourse.getElectiveCourseId()); //选修课Id
            queryWrapper.eq(ElectiveEnroll::getYn,ChooseStatus.NORMAL.getValue());
            queryWrapper.orderByDesc(ElectiveEnroll::getCreateTime);
           // PageHelper.startPage(electiveCourse.getPageNumber(),electiveCourse.getPageSize());

            Page<ElectiveEnroll> page = new Page<>(electiveCourse.getPageNumber(),electiveCourse.getPageSize());

            IPage<ElectiveEnroll> list =electiveEnrollService.page(page,queryWrapper);
            List<ElectiveEnroll> records = list.getRecords();
            for (ElectiveEnroll record : records) {
                Date date01 = DateUtil.localDateTime2Date(record.getCreateTime());
                String date1 = DateUtil.dateToString(date01);
                record.setStringCreateTime(date1);
            }
            // List<ElectiveEnroll> list = electiveEnrollService.list(queryWrapper);
          //  PageInfo<ElectiveEnroll> pageInfo=new PageInfo<>(list);
            log.info("*******list******, :{}",list.toString());
            return ResponseModel.success(list).code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("*****e***e***e**,:{}",e.getMessage());
            return ResponseModel.failure("查询报名列表失败").code(ResultCode.ERROR);
        }


    }

}

