package com.ipsa.learn.controller;

import com.dingtalk.api.response.*;
import com.ipsa.learn.entity.*;
import com.ipsa.learn.service.CrsPGatewayInfoService;
import com.ipsa.learn.service.CrsPOrderFormService;
import com.ipsa.learn.service.CrsPPayRecordService;
import com.ipsa.learn.util.*;
import com.ipsa.learn.util.dingtalkUtil.CssDingUtil;
import com.taobao.api.ApiException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.*;

/**
 * @author Mir.Li
 * @date 2020/8/19  -  21:27
 */
@Slf4j
@Api(tags = "钉钉开发文档")
@RestController
@RequestMapping("/wechat")
public class WechatController extends BaseController{

    @Value("${idcompany}")
    private String idCompany;

    @Value("${channelid}")
    private String channelId;

    @Value("${isdev}")
    private String isDev;

    @Value("${servernotify}")
    private String serverNotify;


    @Autowired
    CrsPOrderFormService crsPOrderFormService;

    @Autowired
    CrsPPayRecordService crsPPayRecordService;

    @Autowired
    CrsPGatewayInfoService crsPGatewayInfoService;

    @Autowired
    IdWorker idWorker;

    @Autowired
    MsgManager msgManager;

    @ApiOperation("获取用户的详细信息")
    @GetMapping("/getUerInfo/{authCode}")
    public ResponseModel<OapiUserGetResponse> getUerInfo(@PathVariable("authCode") String authCode){
        OapiUserGetResponse schoolName = CssDingUtil.getUerInfo(authCode);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("获取用户Id")
    @GetMapping("/getUserId/{authCode}")
    public ResponseModel<String> getUserId(@PathVariable("authCode") String authCode){
        String schoolName = CssDingUtil.getUserId(authCode);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据用户Id查询用户角色")
    @GetMapping("/getClassParentsList/{userId}")
    public ResponseModel<OapiEduRolesGetResponse> getClassParentsList(@PathVariable("userId") String userId){
        OapiEduRolesGetResponse schoolName = CssDingUtil.getClassParentsList(userId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级id 查询家长列表")
    @GetMapping("/getClassParentsList/{classId}/{pageNo}/{pageSize}")
    public ResponseModel<OapiEduGuardianListResponse> getClassParentsList(@PathVariable("classId") String classId,@PathVariable("pageNo") String pageNo,@PathVariable("pageSize") String pageSize){
        OapiEduGuardianListResponse schoolName = CssDingUtil.getClassParentsList(Long.parseLong(classId),Long.parseLong(pageNo),Long.parseLong(pageSize));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级id 家长Id查找家长信息")
    @GetMapping("/getClassParents/{classId}/{guarDianUserId}")
    public ResponseModel<OapiEduGuardianGetResponse> getClassParents(@PathVariable("classId") String classId,@PathVariable("guarDianUserId") String guarDianUserId){
        OapiEduGuardianGetResponse schoolName = CssDingUtil.getClassParents(Long.parseLong(classId),guarDianUserId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级id 查找老师列表")
    @GetMapping("/getClassTeacherList/{classId}")
    public ResponseModel<OapiEduTeacherListResponse> getClassTeacherList(@PathVariable("classId") String classId){
        OapiEduTeacherListResponse schoolName = CssDingUtil.getClassTeacherList(Long.parseLong(classId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级id 用户Id查找老师信息")
    @GetMapping("/getClassTeacher/{classId}/{studentUserId}")
    public ResponseModel<OapiEduTeacherGetResponse> getClassTeacher(@PathVariable("classId") String classId,@PathVariable("teacherUserId") String teacherUserId){
        OapiEduTeacherGetResponse schoolName = CssDingUtil.getClassTeacher(Long.parseLong(classId),teacherUserId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级id查询学生列表")
    @GetMapping("/getStudentList/{classId}/{pageNo}/{pageSize}")
    public ResponseModel<OapiEduStudentListResponse> getStudentList(@PathVariable("classId") String classId,@PathVariable("pageNo") String pageNo,@PathVariable("pageSize") String pageSize){
        OapiEduStudentListResponse schoolName = CssDingUtil.getStudentList(Long.parseLong(classId),Long.parseLong(pageNo),Long.parseLong(pageSize));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级id,学生id，查找学生信息")
    @GetMapping("/getStudentName/{classId}/{studentUserId}")
    public ResponseModel<OapiEduStudentGetResponse> getStudentName(@PathVariable("classId") String classId,@PathVariable("studentUserId") String studentUserId){
        OapiEduStudentGetResponse schoolName = CssDingUtil.getStudentName(Long.parseLong(classId),studentUserId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据班级Id查询班级信息")
    @GetMapping("/getEduInFormAtion/{classId}")
    public ResponseModel<OapiEduClassGetResponse> getEduInFormAtion(@PathVariable("classId") String classId){
        OapiEduClassGetResponse schoolName = CssDingUtil.getEduInFormAtion(Long.parseLong(classId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("根据年级Id查询班级列表")
    @GetMapping("/getEduPeriodClass/{gradeId}/{pageNo}/{pageSize}")
    public ResponseModel<OapiEduClassListResponse> getEduPeriodClass(@PathVariable("gradeId") String gradeId,@PathVariable("pageNo") String pageNo,@PathVariable("pageSize") String pageSize){
        OapiEduClassListResponse schoolName = CssDingUtil.getEduPeriodClass(Long.parseLong(gradeId),Long.parseLong(pageNo),Long.parseLong(pageSize));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("学段Id查询年级信息")
    @GetMapping("/getEduPeriodGrade/{periodId}")
    public ResponseModel<OapiEduGradeGetResponse> getEduPeriodGrade(@PathVariable("periodId") String periodId){
        OapiEduGradeGetResponse schoolName = CssDingUtil.getEduPeriodGrade(Long.parseLong(periodId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("学段Id查询年级列表")
    @GetMapping("/getEduPeriodLearning/{periodId}")
    public ResponseModel<OapiEduGradeListResponse> getEduPeriodLearning(@PathVariable("periodId") String periodId){
        OapiEduGradeListResponse schoolName = CssDingUtil.getEduPeriodLearning(Long.parseLong(periodId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("查询学段信息")
    @GetMapping("/getEduPeriodLearningText/{periodId}")
    public ResponseModel<OapiEduPeriodGetResponse> getEduPeriodLearningText(@PathVariable("periodId") String periodId){
        OapiEduPeriodGetResponse schoolName = CssDingUtil.getEduPeriodLearningText(Long.parseLong(periodId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("查询学段列表")
    @GetMapping("/getEduLearningPeriod/{campusId}")
    public ResponseModel<OapiEduPeriodListResponse> getEduLearningPeriod(@PathVariable("campusId") String campusId){
        OapiEduPeriodListResponse schoolName = CssDingUtil.getEduLearningPeriod(Long.parseLong(campusId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @ApiOperation("查询校区信息")
    @GetMapping("/getschool/{campusId}")
    public ResponseModel<OapiEduCampusGetResponse> getSchool(@PathVariable("campusId") String campusId){
        OapiEduCampusGetResponse schoolName = CssDingUtil.getSchoolText(Long.parseLong(campusId));
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @GetMapping("/getschoolname")
    public ResponseModel<String> getSchoolName(){
        String schoolName = CssDingUtil.getSchoolName();
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }

    @GetMapping("/getdeportment")
    public ResponseModel<String> getDeportment(){
        String department = CssDingUtil.getDepartment();
        return ResponseModel.success(department).code(ResultCode.SUCCESS);
    }

    @PostMapping("/weixin")
    public ResponseModel<Map<String,String>> wechatRedirect(@RequestBody CrsPOrderForm crsPOrderForm){
        log.info("------------开始微信统一下单接口调用---------------");
        Date currDate = new Date();
        CrsPOrderForm orderForm = null;
        try {
            String orderNo = Long.toString(idWorker.nextId());
            String payMethod = ParameterStatus.PAY_METHOD_WECHAT;
            //创建订单
            crsPOrderForm.setOrderId(UUIDGenerator.getUUIDs());
            crsPOrderForm.setOrderNo(orderNo);
            crsPOrderForm.setPayMethod(payMethod);
            crsPOrderForm.setUpdatedBy("SYSTEM");
            //前期写死
            crsPOrderForm.setIdCompany(idCompany);
            crsPOrderForm.setChannelId(channelId);

            crsPOrderFormService.save(crsPOrderForm);
            List<CrsPOrderForm> pOrderForms = crsPOrderFormService.getCrsOrderFrom(orderNo);
            orderForm = pOrderForms.get(0);
            // 查询渠道对应的财付通商户信息
            ChannelGatewayModel model = new ChannelGatewayModel();
            model.setChannelId(orderForm.getChannelId());
            model.setPayMethod(payMethod);
            model.setIdCompany(orderForm.getIdCompany());
            List<CrsPGatewayInfo> crsPGatewayInfo = crsPGatewayInfoService.getGatewayInfo(model);
            if (CollectionUtils.isEmpty(crsPGatewayInfo)) {
                // 处理网关信息丢失的异常
                // 发送短信
                String content = "微信支付网关：" + payMethod + ",渠道id："
                        + orderForm.getChannelId() + ",渠道没有配置网关信息，请配置对应的收款账号信息";
                msgManager.sendMsg(content);
            }
            //crsPGatewayInfo
            CrsPGatewayInfo gatewayInfo = crsPGatewayInfo.get(0);
            String nonceStr = Sha1Util.getNonceStr();

            TreeMap<String, String> signParams = new TreeMap<String,String>();
            signParams.put("appid",gatewayInfo.getAppId()); //公众号Id
            signParams.put("mch_id",gatewayInfo.getBusCoopId());  //商户号
            signParams.put("nonce_str",nonceStr);
            signParams.put("body",orderForm.getSchoolName());  //商品描述
            signParams.put("out_trade_no",orderForm.getOrderNo());  //商户订单号
            String total_fee = "" + orderForm.getOrderAmount().multiply(new BigDecimal(100)).intValue();
            if (Boolean.parseBoolean(isDev)){
                total_fee = "1";
            }
            signParams.put("total_fee",total_fee);  //总金额
            signParams.put("trade_type","MWEB");  //交易类型
            signParams.put("product_id",orderForm.getOrderNo());  //商品Id
            signParams.put("spbill_create_ip",request.getRemoteAddr()); //IP
            signParams.put("notify_url",serverNotify+"/wechat/notify");

            String payExpire = orderForm.getPayExpire();
            if(StringUtils.isNotBlank(payExpire)){
                signParams.put("time_start", DateUtils.formatDate(currDate, "yyyyMMddHHmmss"));
                signParams.put("time_expire", DateUtils.formatDate(com.ipsa.learn.util.DateUtils.getDate(currDate, Long.parseLong(payExpire) * 60 * 1000), "yyyyMMddHHmmss"));
            }
            String signString = SignCheck.createLinkString(signParams);
            String paySignKey = gatewayInfo.getPaySignKey(); //商户秘钥
            log.info("*******paySignKey***商户秘钥*** :{}",paySignKey);
            String sign = WxSign.sign(signString, gatewayInfo.getPaySignKey(), "UTF-8");
            signParams.put("sign", sign);
            //String xmlParam = WXPayUtil.generateSignedXml(signParams, paySignKey);
            //System.out.println(xmlParam);
            String reqXml = XMLUtil.parseToXML(signParams);
            String parm = WechatConfig.WECHAT_UNIFIEDORDER + "?" + reqXml;
            log.info(parm);
            //订单的支付记录
            CrsPPayRecord payRecord = new CrsPPayRecord();
            payRecord.setPayOrderNo(UUIDGenerator.getUUID());
            payRecord.setOrderNo(orderNo);
            payRecord.setPayMethod(payMethod);
            payRecord.setCreatedBy("SYSTEM");
            payRecord.setPayStatus("WAIT_BUYER_PAY");
            payRecord.setCreatedTime(DateUtil.DateToLocalDateTime());
            crsPPayRecordService.save(payRecord);
            //发送预订单请求
            String resultXml = HttpRequestHelper.requestByPost(WechatConfig.WECHAT_UNIFIEDORDER, "UTF-8", reqXml);
            log.info(resultXml);
            /*HttpClient client = new HttpClient("https://api.mch.weixin.qq.com/pay/unifiedorder");
            client.setHttps(true);
            client.setXmlParam(xmlParam);
            client.post();
            String result = client.getContent();
            System.out.println(result);*/
            //Map<String, String> resultMap = WXPayUtil.xmlToMap(result);
            Map<String,Object> resultMap = XMLUtil.doXMLParse(resultXml);
            log.info(resultMap.toString());
            String prepay_id = "";
            String mweb_url = "";
            String return_msg = "";
            Map<String, String> map = new HashMap<String, String>();
            if("SUCCESS".equals(resultMap.get("return_code")) && "SUCCESS".equals(resultMap.get("result_code"))){
                prepay_id = resultMap.get("prepay_id").toString();
                mweb_url = resultMap.get("mweb_url").toString();
                // 订单支付记录
                map.put("is_success", "true");
                //支付的交易号,微信的prepay_id
                map.put("prepay_id", prepay_id);
                //支付的链接地址
                map.put("mweb_url",mweb_url);
                map.put("total_fee", orderForm.getOrderAmount().toString()); //总金额
            }else{
                map.put("is_success", "false");
                return_msg = resultMap.get("return_msg").toString();
                map.put("return_msg",return_msg);
            }
            map.put("channel_id", orderForm.getChannelId());
            map.put("out_trade_no", orderForm.getOrderNo());
            return ResponseModel.success(map).code(ResultCode.SUCCESS);
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage());
        } catch (IOException e) {
            String errMsg = "Problem redirecting to wechat: " + e.toString();
            log.info(errMsg);
            // 处理调用支付宝网关异常的情况
            // 发送短信
            String content = orderForm.getOrderNo() + "调用微信网关失败，请检查系统网络是否畅通";
            try {
                msgManager.sendMsg(content);
            } catch (Exception e1) {
                log.error(e1.getMessage());
                e1.printStackTrace();
            }
            return ResponseModel.failure("****调用微信支付端口失败****").code(ResultCode.ERROR);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return null;
    }
    @ApiOperation("根据用户Id获取部门详情")
    @GetMapping("/getDepartment/{id}")
    public ResponseModel<OapiDepartmentGetResponse > getDepartment(@PathVariable("id") String userId){
        OapiDepartmentGetResponse  schoolName = CssDingUtil.getDepartment(userId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }
    @ApiOperation("根据用户userId查询指定用户的所有上级父部门路径")
    @GetMapping("/getListParentDept/{userId}")
    public ResponseModel<OapiDepartmentListParentDeptsResponse> getListParentDept(@PathVariable("userId") String userId){
        OapiDepartmentListParentDeptsResponse  schoolName = CssDingUtil.getListParentDept(userId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }
    @ApiOperation("根据部门的id查询部门的所有上级父部门路径")
    @GetMapping("/getListParentByDept/{id}")
    public ResponseModel<OapiDepartmentListParentDeptsByDeptResponse> getListParentByDept(@PathVariable("id") String id){
        OapiDepartmentListParentDeptsByDeptResponse  schoolName = CssDingUtil.getListParentByDept(id);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }
    @ApiOperation("查看班级下某个监护人的人员关系列表。")
    @GetMapping("/getClassParentsList/{classId}/{fromUserId}")
    public ResponseModel<OapiEduUserRelationGetResponse> getClassParentsList(@PathVariable("classId") String classId,@PathVariable("fromUserId") String fromUserId){
        OapiEduUserRelationGetResponse schoolName = CssDingUtil.getRelation(Long.parseLong(classId),fromUserId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }


    @ApiOperation("用户信息组装")
    @GetMapping("/getUerInfoDetail/{authCode}")
    public ResponseModel<List<UserVo>> getUerInfoDetail(@PathVariable("authCode") String authCode) throws ApiException {
        List<UserVo> UserList = new ArrayList<>();
        UserVo UserVo=CssDingUtil.getUerInfoDetail(authCode);
        UserList.add(UserVo);
        return ResponseModel.success(UserList).code(ResultCode.SUCCESS);
    }
    @ApiOperation("用户信息组装")
    @GetMapping("/getUerInfoDetailByUserId/{userId}")
    public ResponseModel<List<UserVo>> getUerInfoDetailByUserId(@PathVariable("userId") String userId) throws ApiException {
        List<UserVo> UserList = new ArrayList<>();
        UserVo UserVo=CssDingUtil.getUerInfoDetailByUserId(userId);
        UserList.add(UserVo);
        return ResponseModel.success(UserList).code(ResultCode.SUCCESS);
    }
    @ApiOperation("H5获取学校信息")
    @GetMapping("/getSchoolDetail")
    public ResponseModel<List<UserVo>> getSchoolDetail() throws ApiException {
        List<UserVo> UserList = new ArrayList<>();
        UserVo UserVo=CssDingUtil.getSchoolDetail();
        UserList.add(UserVo);
        return ResponseModel.success(UserList).code(ResultCode.SUCCESS);
    }
    @ApiOperation("测试查看个人信息")
    @GetMapping("/getUserIdsList/{userId}")
    public ResponseModel<OapiUserGetResponse> getUserIdsList(@PathVariable("userId") String userId){
        OapiUserGetResponse schoolName = CssDingUtil.getUserIdsList(userId);
        return ResponseModel.success(schoolName).code(ResultCode.SUCCESS);
    }
    @ApiOperation("根据年级id获取所有班级")
    @GetMapping("/getEduPeriodClassAll/{gradeId}")
    public ResponseModel<List<OapiEduClassListResponse.ClassResponse>> getEduPeriodClassAll(@PathVariable("gradeId") String gradeId){
        List<OapiEduClassListResponse.ClassResponse>  getEduPeriodClassAll = CssDingUtil.getEduPeriodClassAll(Long.parseLong(gradeId));
        return ResponseModel.success(getEduPeriodClassAll).code(ResultCode.SUCCESS);
    }
    @ApiOperation("根据班级id获取所有学生")
    @GetMapping("/getStudentListAll/{classId}")
    public ResponseModel<List<OapiEduStudentListResponse.StudentRespone>> getStudentListAll(@PathVariable("classId") String classId){
        List<OapiEduStudentListResponse.StudentRespone>  getStudentListAll = CssDingUtil.getStudentListAll(Long.parseLong(classId));
        return ResponseModel.success(getStudentListAll).code(ResultCode.SUCCESS);
    }
}
