package com.ipsa.learn.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ipsa.learn.entity.CourseApplication;
import com.ipsa.learn.dao.CourseApplicationDao;
import com.ipsa.learn.entity.CourseDailycheck;
import com.ipsa.learn.entity.CurriculumDTO;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.CourseApplicationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ipsa.learn.util.ChooseStatus;
import com.ipsa.learn.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 代课申请 服务实现类
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-31
 */
@Service
public class CourseApplicationServiceImpl extends ServiceImpl<CourseApplicationDao, CourseApplication> implements CourseApplicationService {

    @Autowired
    CourseApplicationService courseApplicationService;

    @Override
    public List<CourseApplication> getCourseApplication(ScheduleDto scheduleDto) {
        LambdaQueryWrapper<CourseApplication> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolCodeDto())){
            //学校编码
            queryWrapper.eq(CourseApplication::getSchoolCode,scheduleDto.getSchoolCodeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolYearDto())){
            //学年
            queryWrapper.eq(CourseApplication::getSchoolYear,scheduleDto.getSchoolYearDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSemesterDto())){
            //学期
            queryWrapper.eq(CourseApplication::getSemester,scheduleDto.getSemesterDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getGradeDto())){
            //年级
            queryWrapper.eq(CourseApplication::getGrade,scheduleDto.getGradeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getClassCodeDto())){
            //班级
            queryWrapper.eq(CourseApplication::getClassCode,scheduleDto.getClassCodeDto());
        }
        if (null != scheduleDto.getStatusDto()){
            //审核通过
            queryWrapper.eq(CourseApplication::getStatus,scheduleDto.getStatusDto());
        }
        /*if (StringUtils.isNotEmpty(scheduleDto.getApplicationNameDto())){
            //申请人
            queryWrapper.eq(CourseApplication::getApplicationName,scheduleDto.getApplicationNameDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getCurriculumNameDto())){
            //课程名称
            queryWrapper.like(CourseApplication::getCurriculumName,scheduleDto.getCurriculumNameDto());
        }*/
        if (scheduleDto.getStartTimeDto() != null && scheduleDto.getLastTimeDto() != null){
            //时间范围
            queryWrapper.between(CourseApplication::getApplicationTime,scheduleDto.getStartTimeDto(),scheduleDto.getLastTimeDto());
        }
        //时间倒序
        queryWrapper.orderByDesc(CourseApplication::getCreateTime);
        queryWrapper.eq(CourseApplication::getYn, ChooseStatus.NORMAL.getValue()); //正常
        List<CourseApplication> list = courseApplicationService.list(queryWrapper);
        for (CourseApplication courseApplication : list) {
            Date date01 = DateUtil.localDateTime2Date(courseApplication.getCreateTime());
            Date date02 = DateUtil.localDateTime2Date(courseApplication.getApplicationTime());
            String date1 = DateUtil.dateToString(date01);
            String date2 = DateUtil.dateToString(date02);
            courseApplication.setStringCreateTime(date1);
            courseApplication.setStringApplicationTime(date2);
        }
        return list;
    }

    /**
     * 代课审核校验查询接口
     * @param curriculumDTO
     * @return
     */
    @Override
    public List<CourseApplication> getCheckCourseApplication(CurriculumDTO curriculumDTO) {
        LambdaQueryWrapper<CourseApplication> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(curriculumDTO.getSchoolCode())){
            //学校编码
            queryWrapper.eq(CourseApplication::getSchoolCode,curriculumDTO.getSchoolCode());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getSchoolYear())){
            //学年
            queryWrapper.eq(CourseApplication::getSchoolYear,curriculumDTO.getSchoolYear());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getSemester())){
            //学期
            queryWrapper.eq(CourseApplication::getSemester,curriculumDTO.getSemester());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getGrade())){
            //年级
            queryWrapper.eq(CourseApplication::getGrade,curriculumDTO.getGrade());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getClassCode())){
            //班级
            queryWrapper.eq(CourseApplication::getClassCode,curriculumDTO.getClassCode());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getWeekDto())){
            //周几
            queryWrapper.eq(CourseApplication::getWeek,curriculumDTO.getWeekDto());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getStringClassTimeDto())){
            //申请代课的时间
            LocalDateTime localDateTime = LocalDateTime.parse(curriculumDTO.getStringClassTimeDto()+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            queryWrapper.eq(CourseApplication::getApplicationTime,localDateTime);
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getClassPeriod())){
            //课节
            queryWrapper.eq(CourseApplication::getClassPeriod,curriculumDTO.getClassPeriod());
        }
        List<CourseApplication> list = courseApplicationService.list(queryWrapper);
        return list;
    }
}
