package com.ipsa.learn.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ipsa.learn.entity.*;
import com.ipsa.learn.dao.CourseDailycheckDao;
import com.ipsa.learn.service.CourseDailycheckService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ipsa.learn.util.ChooseStatus;
import com.ipsa.learn.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 日常检查 服务实现类
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-09-03
 */
@Slf4j
@Service
public class CourseDailycheckServiceImpl extends ServiceImpl<CourseDailycheckDao, CourseDailycheck> implements CourseDailycheckService {

    @Autowired
    CourseDailycheckService courseDailycheckService;

    @Override
    public List<CourseDailycheck> getDailycheckList(ScheduleDto scheduleDto) {
        LambdaQueryWrapper<CourseDailycheck> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolCodeDto())){
            //学校编码
            queryWrapper.eq(CourseDailycheck::getSchoolCode,scheduleDto.getSchoolCodeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolYearDto())){
            //学年
            queryWrapper.eq(CourseDailycheck::getSchoolYear,scheduleDto.getSchoolYearDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSemesterDto())){
            //学期
            queryWrapper.eq(CourseDailycheck::getSemester,scheduleDto.getSemesterDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getGradeDto())){
            //年级
            queryWrapper.eq(CourseDailycheck::getGrade,scheduleDto.getGradeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getClassCodeDto())){
            //班级
            queryWrapper.eq(CourseDailycheck::getClassCode,scheduleDto.getClassCodeDto());
        }
        if (scheduleDto.getStartTimeDto() != null && scheduleDto.getLastTimeDto() != null){
            //时间范围
            queryWrapper.between(CourseDailycheck::getCreateTime,scheduleDto.getStartTimeDto(),scheduleDto.getLastTimeDto());
        }
        queryWrapper.eq(CourseDailycheck::getYn, ChooseStatus.NORMAL.getValue());
        queryWrapper.orderByDesc(CourseDailycheck::getCreateTime);
        List<CourseDailycheck> courseDailychecks = courseDailycheckService.list(queryWrapper);
        for (CourseDailycheck courseDailycheck : courseDailychecks) {
            Date date = DateUtil.localDateTime2Date(courseDailycheck.getCreateTime());
            String date1 = DateUtil.dateToStrings(date);
            courseDailycheck.setStringCreateTime(date1);
            //System.out.println(date1);
        }

        return courseDailychecks;
    }



    @Override
    public List<RedFlag> getRedFlag(RedFlagRequest request) {
        return baseMapper.selectRedFlag(request);
    }

    @Override
    public List<RedFlagScore> getRedFlagScore(RedFlagRequest request) {
        return baseMapper.selectRedFlagScore(request);
    }
}
