package com.ipsa.learn.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ipsa.learn.entity.CourseExamination;
import com.ipsa.learn.dao.CourseExaminationDao;
import com.ipsa.learn.entity.CourseTransfer;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.CourseExaminationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ipsa.learn.service.CourseTransferService;
import com.ipsa.learn.util.ResponseModel;
import com.ipsa.learn.util.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 代课申请 服务实现类
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-09-02
 */
@Slf4j
@Service
public class CourseExaminationServiceImpl extends ServiceImpl<CourseExaminationDao, CourseExamination> implements CourseExaminationService {

    @Autowired
    CourseExaminationService courseExaminationService;

    @Autowired
    CourseTransferService courseTransferService;

    @Override
    public void saveCourseExamination(Long appTranId) {
        log.info("*******ids 审核完成后的id******", appTranId);
        try {
                    CourseTransfer courseTransfer = courseTransferService.getById(appTranId);
                    if (null != courseTransfer){
                        CourseExamination courseExamination = courseExaminationDTO(courseTransfer);
                        courseExaminationService.save(courseExamination);
                        log.info("*******courseExamination*******,:{}",courseExamination.toString());
                        CourseExamination courseExamination1 = courseExaminationNo(courseTransfer);
                        courseExaminationService.save(courseExamination1);
                        log.info("*******courseExamination1*******,:{}",courseExamination1.toString());
                    }
        } catch (Exception e) {
            log.info("****e:*** :{}",e.getMessage());
            e.printStackTrace();
        }
    }

    @Override
    public List<CourseExamination> getCourseExaminationList(ScheduleDto scheduleDto) {
        LambdaQueryWrapper<CourseExamination> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolCodeDto())){
            //学校编码
            queryWrapper.eq(CourseExamination::getSchoolCode,scheduleDto.getSchoolCodeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolYearDto())){
            //学年
            queryWrapper.eq(CourseExamination::getSchoolYear,scheduleDto.getSchoolYearDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSemesterDto())){
            //学期
            queryWrapper.eq(CourseExamination::getSemester,scheduleDto.getSemesterDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getGradeDto())){
            //年级
            queryWrapper.eq(CourseExamination::getGrade,scheduleDto.getGradeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getClassCodeDto())){
            //班级
            queryWrapper.eq(CourseExamination::getClassCode,scheduleDto.getClassCodeDto());
        }

        if (scheduleDto.getStartTimeDto() != null && scheduleDto.getLastTimeDto() != null){
            //时间范围
            queryWrapper.between(CourseExamination::getExaminationTime,scheduleDto.getStartTimeDto(),scheduleDto.getLastTimeDto());
        }
        List<CourseExamination> courseExaminations = courseExaminationService.list(queryWrapper);
        return courseExaminations;
    }

    /**
     * 申请人课程信息
     * @param courseTransfer
     * @return
     */
    private CourseExamination courseExaminationDTO(CourseTransfer courseTransfer){
        CourseExamination courseExamination = new CourseExamination();
        courseExamination.setCourseTransferId(String.valueOf(courseTransfer.getCourseTransferId())); //调课表id
        courseExamination.setSchoolCode(courseTransfer.getSchoolCode()); //学校编码
        courseExamination.setSchoolYear(courseTransfer.getSchoolYear());  //学年
        courseExamination.setSemester(courseTransfer.getSemester());  //学期
        courseExamination.setGrade(courseTransfer.getGrade());  //年级
        courseExamination.setGradeName(courseTransfer.getGradeName());  //年级名称
        courseExamination.setClassCode(courseTransfer.getClassCode());  //班级
        courseExamination.setClassName(courseTransfer.getClassName()); //班级名称
        courseExamination.setClassPeriod(courseTransfer.getClassPeriod());  //课节
        courseExamination.setExaminationTime(courseTransfer.getApplicationTime());  //申请人课程的时间
        courseExamination.setWeek(courseTransfer.getWeek());  //申请人课程的星期几
        courseExamination.setCurriculumName(courseTransfer.getCurriculumName());  //申请人的课程名称
        courseExamination.setReplaceName(courseTransfer.getApplicationName());  //申请调课的申请人的名称
        courseExamination.setCreateUser(courseTransfer.getCreateUser());  //创建人
        courseExamination.setYn(1);
        return courseExamination;
    }

    /**
     * 申请被调课人的信息
     * @return
     */
    private CourseExamination courseExaminationNo(CourseTransfer courseTransfer){
        CourseExamination courseExamination = new CourseExamination();
        courseExamination.setCourseTransferId(String.valueOf(courseTransfer.getCourseTransferId())); //调课表id
        courseExamination.setSchoolCode(courseTransfer.getSchoolCode()); //学校编码
        courseExamination.setSchoolYear(courseTransfer.getSchoolYear());  //学年
        courseExamination.setSemester(courseTransfer.getSemester());  //学期
        courseExamination.setGrade(courseTransfer.getGradeGrade());  //年级
        courseExamination.setGradeName(courseTransfer.getGradeGradeName()); //年级名称
        courseExamination.setClassCode(courseTransfer.getClassGrade());  //班级
        courseExamination.setClassName(courseTransfer.getClassGradeName()); //班级名称
        courseExamination.setClassPeriod(courseTransfer.getClassPeriodGrade());  //申请调课人的课节
        courseExamination.setExaminationTime(courseTransfer.getApplicationTimeGrade());  //申请调课人的调课时间
        courseExamination.setWeek(courseTransfer.getWeekGrade()); //申请调课人的星期几
        courseExamination.setCurriculumName(courseTransfer.getCurriculumNameGrade());  //申请调课人的课程名称
        courseExamination.setReplaceName(courseTransfer.getReplaceName()); //申请调课人的名称
        courseExamination.setCreateUser(courseTransfer.getCreateUser()); //创建人
        courseExamination.setYn(1);
        return courseExamination;
    }
}
