package com.ipsa.learn.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ipsa.learn.dao.CommentByStudentDao;
import com.ipsa.learn.entity.*;
import com.ipsa.learn.dao.CourseResultlistDao;
import com.ipsa.learn.entity.dto.UserInfoDto;
import com.ipsa.learn.service.CourseResultlistService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ipsa.learn.service.CourseResultsuploadService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 成绩列表 服务实现类
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-09-04
 */
@Service
public class CourseResultlistServiceImpl extends ServiceImpl<CourseResultlistDao, CourseResultlist> implements CourseResultlistService {

    @Autowired
    CourseResultlistDao courseResultlistDao;

    @Autowired
    CommentByStudentDao commentByStudentDao;

    @Autowired
    CourseResultlistService courseResultlistService;

    @Autowired
    CourseResultsuploadService courseResultsuploadService;
    @Override
    public CourseResultlist getCountSum(String course_resultsupload_id) {
        return courseResultlistDao.getCountSum(course_resultsupload_id);
    }

    @Override
    public List<CourseResultlist> getCourseResult(String courseResultsuploadId) {
        LambdaQueryWrapper<CourseResultlist> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CourseResultlist::getCourseResultsuploadId,courseResultsuploadId);
        queryWrapper.eq(CourseResultlist::getYn,1);
        List<CourseResultlist> resultlist = courseResultlistService.list(queryWrapper);
        return resultlist;
    }

    @Override
    public List<CourseResultlist> getResultList(CourseResultsupload courseResultsupload) {
        String schoolCode = courseResultsupload.getSchoolCode();//学校编码
        String classCode = courseResultsupload.getClassCode();//班级编码
        LambdaQueryWrapper<CourseResultsupload> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CourseResultsupload::getSchoolCode,schoolCode);
        queryWrapper.eq(CourseResultsupload::getClassCode,classCode);
        queryWrapper.eq(CourseResultsupload::getYn,1);
        queryWrapper.orderByDesc(CourseResultsupload::getCreateTime);
        List<CourseResultsupload> list = courseResultsuploadService.list(queryWrapper);
        if (null != list && list.size()>0){
            CourseResultsupload resultsupload = list.get(0);
            String semester = resultsupload.getSemester(); //学期
            String testdateName = resultsupload.getTestdateName(); //考试名称
            List<CourseResultlist> resultList = courseResultlistDao.getResultList(resultsupload);
            return resultList;
        }
        return null;
    }

    @Override
    public List<CourseSoreVo> getParentsList(CourseResultsuploadVo courseResultsuploadVo) {
        String schoolCode = courseResultsuploadVo.getSchoolCode();//学校编码
        String classCode = courseResultsuploadVo.getClassCode();//班级编码
        LambdaQueryWrapper<CourseResultsupload> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CourseResultsupload::getSchoolCode,schoolCode);
        queryWrapper.eq(CourseResultsupload::getClassCode,classCode);
        queryWrapper.orderByDesc(CourseResultsupload::getCreateTime);
        List<CourseResultsupload> list = courseResultsuploadService.list(queryWrapper);
        if (null != list && list.size()>0){
            CourseResultsupload resultsupload = list.get(0);
            String semester = resultsupload.getSemester(); //学期
            String testdateName = resultsupload.getTestdateName(); //考试名称
            courseResultsuploadVo.setSemester(semester);
            courseResultsuploadVo.setTestdateName(testdateName);
            List<CourseSoreVo> parentsList = courseResultlistDao.getParentsList(courseResultsuploadVo);

            return parentsList;
        }
        return null;
    }

    @Override
    public LearnAnalysisRespone getLearnAnalysis(LearnAnalysisRequest request) {

        LearnAnalysisRespone respone = new LearnAnalysisRespone();

        CourseResultlist courseResultlist = baseMapper.selectLearnAnalysis(request);

        if (courseResultlist != null) {

            //年级人总数
            request.setType("1");
            Integer gradeNumber = baseMapper.selectGradeNumber(request);
            //年级人考试总数
            request.setType("2");
            Integer gradeExamNumber = baseMapper.selectGradeNumber(request);

            LearnAnalysisExam exam = new LearnAnalysisExam();
            exam.setClassExamNumber(courseResultlist.getResultsNumbers());
            exam.setClassTotal(courseResultlist.getClassNumbers());
            exam.setGradeExamNumber(gradeExamNumber + "");
            exam.setGradeTotal(gradeNumber + "");

            LearnAnalysis learnAnalysis = new LearnAnalysis();
            learnAnalysis.setANumber(courseResultlist.getVerygoodsNumbers());
            learnAnalysis.setBNumber(courseResultlist.getGoodNumbers());
            learnAnalysis.setCNumber(courseResultlist.getPasstheNumbers());
            learnAnalysis.setDNumber(courseResultlist.getDontpasstheNumbers());
            learnAnalysis.setClassTotal(courseResultlist.getClassNumbers());
            learnAnalysis.setClassExamNumber(courseResultlist.getResultsNumbers());
            learnAnalysis.setGradeExamNumber(gradeExamNumber + "");
            learnAnalysis.setGradeTotal(gradeNumber.toString());


            respone.setAnalysis(learnAnalysis);
            respone.setInfo(exam);
            return respone;
        }
        return null;
    }

    @Override
    public int addCommentByStudent(CommentByStudent dto){
        /**
         *  1,先根据 studenid 和 科目 查询一下，如果不为空，就证明有人评价过了，就执行更新操作
         */

        LambdaQueryWrapper<CommentByStudent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CommentByStudent::getStudentId,dto.getStudentId());
        queryWrapper.eq(CommentByStudent::getSubject,dto.getSubject());
        CommentByStudent commentByStudent = commentByStudentDao.selectOne(queryWrapper);
        if (commentByStudent!=null){ // 执行修改操作
            dto.setId(commentByStudent.getId()); // 把查出来的id,赋值给现在的传来的参数
            dto.setUpdateTime(new Date());  // 修改修改时候
            return commentByStudentDao.updateById(dto);
        }

        dto.setCreateTime(new Date());

        return commentByStudentDao.insert(dto);
    }

    @Override
    public List<CommentByStudent> getCommentByStudent(String studentId){
        LambdaQueryWrapper<CommentByStudent> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CommentByStudent::getStudentId,studentId);
        return commentByStudentDao.selectList(queryWrapper);
    }

    @Override
    public List<StudentTotalScore> getScoreSortingList(RankByClass rankByClass) {
        return commentByStudentDao.getScoreSortingList(rankByClass);
    }

    @Override
    public StudentScoreDetails getStudentScoreDetails(StudentScoreDto studentScoreDto) {
        // 平均分 最高分查询
        StudentScoreDetails avgAndMax = commentByStudentDao.getStudentScoreDetails(studentScoreDto);
        StudentScoreDetails studentScoreDetails = new StudentScoreDetails();
        studentScoreDetails.setAvgScore(avgAndMax.getAvgScore());
        studentScoreDetails.setMaxScore(avgAndMax.getMaxScore());
        //班级排名及分数
        StudentScoreDetails studentScoreRankByClass = commentByStudentDao.getStudentScoreRankByClass(studentScoreDto);
        studentScoreDetails.setScore(studentScoreRankByClass.getScore());
        studentScoreDetails.setClassRank(studentScoreRankByClass.getClassRank());
        studentScoreDetails.setStudentUserId(studentScoreRankByClass.getStudentUserId());
        studentScoreDetails.setStudentUserName(studentScoreRankByClass.getStudentUserName());
        //年级排名
        StudentScoreDetails studentScoreRankByGrade = commentByStudentDao.getStudentScoreRankByGrade(studentScoreDto);
        studentScoreDetails.setGradeRank(studentScoreRankByGrade.getGradeRank());
        //总分查询
        StudentScoreDetails studentTotalScore = commentByStudentDao.getStudentTotalScore(studentScoreDto);
        studentScoreDetails.setTotalScore(studentTotalScore.getTotalScore());

        // 分数表
        List<ScoreTable> scoreTable = commentByStudentDao.getScoreTable(studentScoreDto);
        studentScoreDetails.setScoreTable(scoreTable);
        // 班级排名表
        List<Map<String, Object>> classRankTable = commentByStudentDao.getClassRankTable(studentScoreDto);
        List<ScoreTable> restClassRankTable = new ArrayList<>();
        for (Map it : classRankTable) {
            StudentScoreDto studentScoreDto1 = new StudentScoreDto();
            studentScoreDto1.setClassCode(studentScoreDto.getClassCode());
            studentScoreDto1.setClassSubName(studentScoreDto.getClassSubName());
            studentScoreDto1.setStudentUserId(studentScoreDetails.getStudentUserId());
            studentScoreDto1.setCourseResultsUploadId(it.get("course_resultsupload_id").toString());
            StudentScoreDetails studentScoreRankByClass1 = commentByStudentDao.getStudentScoreRankByClassOne(studentScoreDto1);
            ScoreTable scoreTable1 = new ScoreTable();
            scoreTable1.setClassRank(studentScoreRankByClass1.getClassRank());
            scoreTable1.setTestDateTime(it.get("testdate_time").toString());
            restClassRankTable.add(scoreTable1);
        }
        studentScoreDetails.setClassRankTable(restClassRankTable);

        // 年级排名表
        List<Map<String, Object>> gradeRankTable = commentByStudentDao.getGradeRankTable(studentScoreDto);
        List<ScoreTable> restGradeRankTable = new ArrayList<>();
        for (Map it : gradeRankTable) {
            StudentScoreDto studentScoreDto1 = new StudentScoreDto();
            studentScoreDto1.setGrade(studentScoreDto.getGrade());
            studentScoreDto1.setClassSubName(studentScoreDto.getClassSubName());
            studentScoreDto1.setStudentUserId(studentScoreDetails.getStudentUserId());
            studentScoreDto1.setCourseResultsUploadId(it.get("course_resultsupload_id").toString());
            StudentScoreDetails studentScoreRankByClass1 = commentByStudentDao.getStudentScoreRankByGradeOne(studentScoreDto1);
            ScoreTable scoreTable1 = new ScoreTable();
            scoreTable1.setGradeRank(studentScoreRankByClass1.getGradeRank());
            scoreTable1.setTestDateTime(it.get("testdate_time").toString());
            restGradeRankTable.add(scoreTable1);
        }
        studentScoreDetails.setGradeRankTable(restGradeRankTable);
        return studentScoreDetails;
    }

    @Override
    public List<Map<String,String>> getSubjectsName(StudentScoreDto studentScoreDto) {
        return commentByStudentDao.getSubjectsName(studentScoreDto);
    }
}
