package com.ipsa.learn.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ipsa.learn.entity.CourseApplication;
import com.ipsa.learn.entity.CourseTransfer;
import com.ipsa.learn.dao.CourseTransferDao;
import com.ipsa.learn.entity.CurriculumDTO;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.CourseTransferService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ipsa.learn.util.ChooseStatus;
import com.ipsa.learn.util.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 调课申请 服务实现类
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-31
 */
@Service
public class CourseTransferServiceImpl extends ServiceImpl<CourseTransferDao, CourseTransfer> implements CourseTransferService {

    @Autowired
    CourseTransferService courseTransferService;

    @Override
    public List<CourseTransfer> getCourseTransfer(ScheduleDto scheduleDto) {
        LambdaQueryWrapper<CourseTransfer> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolCodeDto())){
            //学校编码
            queryWrapper.eq(CourseTransfer::getSchoolCode,scheduleDto.getSchoolCodeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolYearDto())){
            //学年
            queryWrapper.eq(CourseTransfer::getSchoolYear,scheduleDto.getSchoolYearDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSemesterDto())){
            //学期
            queryWrapper.eq(CourseTransfer::getSemester,scheduleDto.getSemesterDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getGradeDto())){
            //年级
            queryWrapper.eq(CourseTransfer::getGrade,scheduleDto.getGradeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getClassCodeDto())){
            //班级
            queryWrapper.eq(CourseTransfer::getClassCode,scheduleDto.getClassCodeDto());
        }
        if (null != scheduleDto.getStatusDto()){
            //审核状态
            queryWrapper.eq(CourseTransfer::getStatus,scheduleDto.getStatusDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getApplicationNameDto())){
            //申请人
            queryWrapper.eq(CourseTransfer::getApplicationName,scheduleDto.getApplicationNameDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getCurriculumNameDto())){
            //课程名称
            queryWrapper.like(CourseTransfer::getCurriculumName,scheduleDto.getCurriculumNameDto());
        }
        if (scheduleDto.getStartTimeDto() != null && scheduleDto.getLastTimeDto() != null){
            //时间范围
            queryWrapper.between(CourseTransfer::getApplicationTime,scheduleDto.getStartTimeDto(),scheduleDto.getLastTimeDto());
        }
        //时间倒序
        queryWrapper.orderByDesc(CourseTransfer::getCreateTime);
        queryWrapper.eq(CourseTransfer::getYn, ChooseStatus.NORMAL.getValue());
        List<CourseTransfer> courseTransfers = courseTransferService.list(queryWrapper);
        for (CourseTransfer courseTransfer : courseTransfers) {
            Date date01 = DateUtil.localDateTime2Date(courseTransfer.getCreateTime());
            Date date02 = DateUtil.localDateTime2Date(courseTransfer.getApplicationTimeGrade());
            String date1 = DateUtil.dateToString(date01);
            String date2 = DateUtil.dateToString(date02);
            courseTransfer.setStringCreateTime(date1);
            courseTransfer.setStringApplicationTimeGrade(date2);
        }
        return courseTransfers;
    }

    /**
     * 校验调课查询
     * @param curriculumDTO
     * @return
     */
    @Override
    public List<CourseTransfer> getCheckCourseTransfer(CurriculumDTO curriculumDTO) {
        LambdaQueryWrapper<CourseTransfer> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(curriculumDTO.getSchoolCode())){
            //学校编码
            queryWrapper.eq(CourseTransfer::getSchoolCode,curriculumDTO.getSchoolCode());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getSchoolYear())){
            //学年
            queryWrapper.eq(CourseTransfer::getSchoolYear,curriculumDTO.getSchoolYear());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getSemester())){
            //学期
            queryWrapper.eq(CourseTransfer::getSemester,curriculumDTO.getSemester());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getGrade())){
            //年级编码
            queryWrapper.eq(CourseTransfer::getGrade,curriculumDTO.getGrade());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getClassCode())){
            //班级编码
            queryWrapper.eq(CourseTransfer::getClassCode,curriculumDTO.getClassCode());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getStringClassTimeDto())){
            //调课的时间
            LocalDateTime localDateTime = LocalDateTime.parse(curriculumDTO.getStringClassTimeDto()+" 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            queryWrapper.eq(CourseTransfer::getApplicationTime,localDateTime);
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getWeekDto())){
            //周几
            queryWrapper.eq(CourseTransfer::getWeek,curriculumDTO.getWeekDto());
        }
        if (StringUtils.isNotEmpty(curriculumDTO.getClassPeriod())){
            //课节
            queryWrapper.eq(CourseTransfer::getClassPeriod,curriculumDTO.getClassPeriod());
        }
        List<CourseTransfer> list = courseTransferService.list(queryWrapper);
        return list;
    }
}
