package com.ipsa.learn.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ipsa.learn.dao.CurriculumDao;
import com.ipsa.learn.entity.Curriculum;
import com.ipsa.learn.entity.ScheduleDto;
import com.ipsa.learn.service.CurriculumService;
import com.ipsa.learn.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * <p>
 * 排课表 服务实现类
 * </p>
 *
 * @author 1079626511@qq.com
 * @since 2020-08-24
 */
@Slf4j
@Service
public class CurriculumServiceImpl extends ServiceImpl<CurriculumDao, Curriculum> implements CurriculumService {


    @Autowired
    CurriculumService curriculumService;

    @Override
    public List<Curriculum> getCurriculum(ScheduleDto scheduleDto) {
        LambdaQueryWrapper<Curriculum> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolCodeDto())){
            //学校编码
            queryWrapper.eq(Curriculum::getSchoolCode,scheduleDto.getSchoolCodeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSchoolYearDto())){
            //学年
            queryWrapper.eq(Curriculum::getSchoolYear,scheduleDto.getSchoolYearDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getSemesterDto())){
            //学期
            queryWrapper.eq(Curriculum::getSemester,scheduleDto.getSemesterDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getGradeDto())){
            //年级
            queryWrapper.eq(Curriculum::getGrade,scheduleDto.getGradeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getClassCodeDto())){
            //班级
            queryWrapper.eq(Curriculum::getClassCode,scheduleDto.getClassCodeDto());
        }
        if (StringUtils.isNotEmpty(scheduleDto.getClassPeriodDto())){
            //课节
            queryWrapper.eq(Curriculum::getClassPeriod,scheduleDto.getClassPeriodDto());
        }
        queryWrapper.eq(Curriculum::getYn, ChooseStatus.NORMAL.getValue());  //排课正常
        queryWrapper.orderByAsc(Curriculum::getClassPeriod);  //按课节进行升序排列
        List<Curriculum> curriculumList = curriculumService.list(queryWrapper);
        return curriculumList;
    }

    @Override
    public ResponseModel<String> uploadExcel(MultipartFile file, ScheduleDto scheduleDto) {
        //String result = "";
        try {
            //创建处理EXCEL的类
            ReadExcel readExcel = new ReadExcel();
            List<Curriculum> curriculumList = readExcel.getExcelInfo(file);
            log.info("*****Excel导入的数据curriculumList****，:{}",curriculumList.toString());
            if (curriculumList.size() == 0 || curriculumList == null){
                return ResponseModel.success("导入的数据为空");
            }
            HashMap<String, Curriculum> curriculumHashMap = new HashMap<>();
            List<Curriculum> curriculum = this.getCurriculum(scheduleDto);
            for (Curriculum curriculum02 : curriculum) {
                //将从数据库中查出来的数据放入hashmap中
                curriculumHashMap.put(curriculum02.getClassPeriod(),curriculum02);
            }
            //遍历导入的数据,如果Excel中的数据在mysql中存在则删除后再添加，如果不存在则添加
            for (Curriculum curriculum01 : curriculumList) {
                if (curriculumHashMap.get(curriculum01.getClassPeriod()) != null){
                    Curriculum curriculum03 = curriculumHashMap.get(curriculum01.getClassPeriod());
                    curriculum03.setYn(0);
                    curriculumService.updateById(curriculum03);  //删除完成之后在添加
                    curriculum01.setSchoolCode(scheduleDto.getSchoolCodeDto());
                    curriculum01.setSchoolYear(scheduleDto.getSchoolYearDto());
                    curriculum01.setSemester(scheduleDto.getSemesterDto());
                    curriculum01.setGrade(scheduleDto.getGradeDto());
                    curriculum01.setClassCode(scheduleDto.getClassCodeDto());
                    curriculum01.setSchoolName(scheduleDto.getSchoolNameDto());
                    curriculum01.setGradeName(scheduleDto.getGradeNameDto());
                    curriculum01.setClassName(scheduleDto.getClassNameDto());
                    curriculum01.setCreateUser(scheduleDto.getCreateUserDto());
                    curriculum01.setYn(1);
                    //curriculum01.setCreateTime(DateUtil.DateToLocalDateTime());
                    curriculumService.save(curriculum01);
                }else {
                    //数据库中没有则直接添加
                    curriculum01.setSchoolCode(scheduleDto.getSchoolCodeDto());
                    curriculum01.setSchoolYear(scheduleDto.getSchoolYearDto());
                    curriculum01.setSemester(scheduleDto.getSemesterDto());
                    curriculum01.setGrade(scheduleDto.getGradeDto());
                    curriculum01.setClassCode(scheduleDto.getClassCodeDto());
                    curriculum01.setSchoolName(scheduleDto.getSchoolNameDto());
                    curriculum01.setGradeName(scheduleDto.getGradeNameDto());
                    curriculum01.setClassName(scheduleDto.getClassNameDto());
                    curriculum01.setCreateUser(scheduleDto.getCreateUserDto());
                    curriculum01.setYn(1);
                    //curriculum01.setCreateTime(DateUtil.DateToLocalDateTime());
                    curriculumService.save(curriculum01);
                }
            }
            return ResponseModel.success("导入成功").code(ResultCode.SUCCESS);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseModel.failure("导入失败").code(ResultCode.ERROR);
        }
    }

    @Override
    public List<String> getSubjectQuery(ScheduleDto scheduleDto) {
        List<Curriculum> curriculum = getCurriculum(scheduleDto);
        log.info("*****curriculum***** :{}",curriculum.toString());

        Map<String, String> map = new HashMap<>();
        for (Curriculum curriculum1 : curriculum) {
            map.put(curriculum1.getMondayCurriculum(),curriculum1.getMondayCurriculum());
            map.put(curriculum1.getTuesdayCurriculum(),curriculum1.getTuesdayCurriculum());
            map.put(curriculum1.getWednesdayCurriculum(),curriculum1.getWednesdayCurriculum());
            map.put(curriculum1.getThursdayCurriculum(),curriculum1.getThursdayCurriculum());
            map.put(curriculum1.getFridayCurriculum(),curriculum1.getFridayCurriculum());
        }
        List<String> list = new ArrayList<>(map.keySet());
        return list;
    }
}
