package com.ipsa.learn.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Calendar;
import java.util.Date;

/**
 * @author Mir.Li
 * @date 2020/8/19  -  0:05
 */
public class DateUtil {

    /**
     * 获取上周五时间
     */
    public static Date lastFirday() {
        //作用防止周日得到本周日期
        Calendar calendar = Calendar.getInstance();
        while (calendar.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY) {
            calendar.add(Calendar.DAY_OF_WEEK, -1);
        }
        int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        int offset = 7 - dayOfWeek;
        calendar.add(Calendar.DATE, offset - 9);

        return DateUtil.getFirstDayOfWeek(calendar.getTime(), 6);//这是从上周日开始数的到本周五为6

    }

    /**
     * 得到某一天的该星期的第一日 00:00:00
     *
     * @param date
     * @param firstDayOfWeek
     *            一个星期的第一天为星期几
     *
     * @return
     */
    public static Date getFirstDayOfWeek(Date date, int firstDayOfWeek) {
        Calendar cal = Calendar.getInstance();
        if (date != null)
            cal.setTime(date);
        cal.setFirstDayOfWeek(firstDayOfWeek);//设置一星期的第一天是哪一天
        cal.set(Calendar.DAY_OF_WEEK, firstDayOfWeek);//指示一个星期中的某天
        cal.set(Calendar.HOUR_OF_DAY, 0);//指示一天中的小时。HOUR_OF_DAY 用于 24 小时制时钟。例如，在 10:04:15.250 PM 这一时刻，HOUR_OF_DAY 为 22。
        cal.set(Calendar.MINUTE, 0);//指示一小时中的分钟。例如，在 10:04:15.250 PM 这一时刻，MINUTE 为 4。
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTime();
    }



    /**
     * 获取上周一时间
     */
    public static Date lastMonday() {
        Calendar calendar = Calendar.getInstance();
        while (calendar.get(Calendar.DAY_OF_WEEK) != Calendar.MONDAY) {
            calendar.add(Calendar.DAY_OF_WEEK, -1);
        }
        int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK) - 1;
        int offset = 1 - dayOfWeek;
        calendar.add(Calendar.DATE, offset - 7);
        return DateUtil.getFirstDayOfWeek(calendar.getTime(), 2);
    }

    public static Date DateTransfer(Date date) throws ParseException {
        SimpleDateFormat dateformat = new SimpleDateFormat("yyyy-MM-dd");
        String format = dateformat.format(date);
        Date parse = dateformat.parse(format);

        return parse;
    }



    public static String dateToString_exact(Date date){
        SimpleDateFormat dateFormat=new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        String now = dateFormat.format(date);
        return now;
    }

    public static String dateToString(Date date){
        SimpleDateFormat dateFormat=new SimpleDateFormat("yyyy-MM-dd");
        String now = dateFormat.format(date);
        return now;
    }

    public static String dateToStrings(Date date){
        SimpleDateFormat dateFormat=new SimpleDateFormat("yyyy-MM-dd");
        String now = dateFormat.format(date);
        return now;
    }


    public static String currentSystemTime(){
        SimpleDateFormat dateformat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dateStr = dateformat.format(System.currentTimeMillis());
        return dateStr;
    }


    public static Date stringToDate(String date) throws ParseException {
        SimpleDateFormat dateformat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        Date parse = dateformat.parse(date);

        return parse;
    }

    public static LocalDateTime DateToLocalDateTime(){
            Instant instant = new Date().toInstant();//An instantaneous point on the time-line.(时间线上的一个瞬时点。)
            ZoneId zoneId = ZoneId.systemDefault();//A time-zone ID, such as {@code Europe/Paris}.(时区)
            LocalDateTime localDateTime = instant.atZone(zoneId).toLocalDateTime();
            return localDateTime;
            //System.out.println(localDateTime.toString());//2018-03-27T14:07:32.668
            //System.out.println(localDateTime.toLocalDate() + " " +localDateTime.toLocalTime());//2018-03-27 14:48:57.453

            //DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");//This class is immutable and thread-safe.@since 1.8
            //System.out.println(dateTimeFormatter.format(localDateTime));//2018-03-27 14:52:57
    }

    /**
     * LocalDateTime转换为Date
     * @param localDateTime
     */
    public static Date localDateTime2Date( LocalDateTime localDateTime) {
        ZoneId zoneId = ZoneId.systemDefault();
        ZonedDateTime zdt = localDateTime.atZone(zoneId);//Combines this date-time with a time-zone to create a  ZonedDateTime.
        Date date = Date.from(zdt.toInstant());
        System.out.println(date.toString());//Tue Mar 27 14:17:17 CST 2018
        return date;
    }

    public static String localDateTime3Date( LocalDateTime localDateTime) {
        ZoneId zoneId = ZoneId.systemDefault();
        ZonedDateTime zdt = localDateTime.atZone(zoneId);//Combines this date-time with a time-zone to create a  ZonedDateTime.
        Date date = Date.from(zdt.toInstant());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String format1 = format.format(date);
        System.out.println(format1);//Tue Mar 27 14:17:17 CST 2018
        return format1;
    }

        public static String dateFormate_exact(String date) throws ParseException {
        if(null!=date && !date.equals("")&&!date.equals("null")){
            SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss.SSSZ");
            SimpleDateFormat sdf1=new SimpleDateFormat("yyyy-MM-dd");
            Date parse = sdf.parse(date);
            String format = sdf1.format(parse);
            return format;
        }else {
            return null;
        }

    }
    public static void main(String[] args) throws ParseException {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = DateUtil.lastMonday();
        String format1 = format.format(date);
        System.out.println(format1);
        Date date1 = DateUtil.lastFirday();
        String format2 = format.format(date1);
        System.out.println(format2);
//        SimpleDateFormat df=new SimpleDateFormat("yyyy-MM-dd");
//        long to =df.parse("2018-04-25").getTime();
//        long from=df.parse("2018-03-24").getTime();
//
//        System.out.println((to-from)/(1000*60*60*24));
       /* boolean equals = "".equals("");
        System.out.println(equals);*/
        //String s = DateUtil.currentSystemTime();
        //System.out.println(s);
        /*LocalDateTime localDateTime = LocalDateTime.now();
        String s = localDateTime3Date(localDateTime);
        System.out.println(s);
        Date date = localDateTime2Date(localDateTime);
        System.out.println(date.toString());*/
    }
}
