package com.ipsa.learn.util;

import com.ipsa.learn.entity.Curriculum;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Mir.Li
 * @date 2020/9/2  -  11:06
 */
public class ReadExcel {
    // 总行数
    private int totalRows = 9;
    // 总条数
    private int totalCells = 0;
    // 错误信息接收器
    private String errorMsg;

    // 构造方法
    public ReadExcel() {
    }

    // 获取总行数
    public int getTotalRows() {
        return totalRows;
    }

    // 获取总列数
    public int getTotalCells() {
        return totalCells;
    }

    // 获取错误信息
    public String getErrorInfo() {
        return errorMsg;
    }

    /**
     * 读EXCEL文件，获取信息集合
     *
     * @param mFile
     * @return
     */
    public List<Curriculum> getExcelInfo(MultipartFile mFile) {
        String fileName = mFile.getOriginalFilename();// 获取文件名
        try {
            if (!validateExcel(fileName)) {// 验证文件名是否合格
                return null;
            }
            boolean isExcel2003 = true;// 根据文件名判断文件是2003版本还是2007版本
            if (isExcel2007(fileName)) {
                isExcel2003 = false;
            }
            List<Curriculum> userList = createExcel(mFile.getInputStream(), isExcel2003);
            return userList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据excel里面的内容读取客户信息
     *
     * @param is
     * @param isExcel2003 excel是2003还是2007版本
     * @return
     * @throws IOException
     */
    public List<Curriculum> createExcel(InputStream is, boolean isExcel2003) {
        try {
            Workbook wb = null;
            if (isExcel2003) {// 当excel是2003时,创建excel2003
                wb = new HSSFWorkbook(is);
            } else {// 当excel是2007时,创建excel2007
                wb = new XSSFWorkbook(is);
            }
            List<Curriculum> userList = readExcelValue(wb);// 读取Excel里面客户的信息
            return userList;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 读取Excel里面客户的信息
     *
     * @param wb
     * @return
     */
    private List<Curriculum> readExcelValue(Workbook wb) {
        // 得到第一个shell
        Sheet sheet = wb.getSheetAt(0);
        // 得到Excel的行数
        //this.totalRows = sheet.getPhysicalNumberOfRows();
        // 得到Excel的列数(前提是有行数)
        if (totalRows > 1 && sheet.getRow(0) != null) {
            this.totalCells = sheet.getRow(1).getPhysicalNumberOfCells();
        }
        List<Curriculum> userList = new ArrayList<Curriculum>();
        // 循环Excel行数
        for (int r = 1; r < totalRows; r++) {
            Row row = sheet.getRow(r);
            if (row == null) {
                continue;
            }
            Curriculum user = new Curriculum();
            // 循环Excel的列
            for (int c = 0; c < this.totalCells; c++) {
                Cell cell = row.getCell(c);
                if (null != cell) {
                    if (c == 0) {
                            String s = String.valueOf(r);
                            user.setClassPeriod(s);// 课节
                    }
                    if (c == 1) {
                        // 如果是纯数字,比如你写的是25,cell.getNumericCellValue()获得是25.0,通过截取字符串去掉.0获得25
                        if (cell.getCellType() == CellType.NUMERIC) {
                            String name = String.valueOf(cell.getNumericCellValue());
                            user.setMondayCurriculum(name.substring(0, name.length() - 2 > 0 ? name.length() - 2 : 1));// 周一科目
                        } else {
                            user.setMondayCurriculum(cell.getStringCellValue());// 周一科目
                        }
                    } else if (c == 2) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String sex = String.valueOf(cell.getNumericCellValue());
                            user.setMondayTeacher(sex.substring(0, sex.length() - 2 > 0 ? sex.length() - 2 : 1));// 周一代课老师
                        } else {
                            user.setMondayTeacher(cell.getStringCellValue());// 周一代课老师
                        }
                    } else if (c == 3) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setTuesdayCurriculum(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周二科目
                        } else {
                            user.setTuesdayCurriculum(cell.getStringCellValue());// 周二科目
                        }
                    } else if (c == 4) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setTuesdayTeacher(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周二代课老师
                        } else {
                            user.setTuesdayTeacher(cell.getStringCellValue());// 周二代课老师
                        }
                    } else if (c == 5) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setWednesdayCurriculum(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周三科目
                        } else {
                            user.setWednesdayCurriculum(cell.getStringCellValue());// 周三科目
                        }
                    } else if (c == 6) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setWednesdayTeacher(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周三代课老师
                        } else {
                            user.setWednesdayTeacher(cell.getStringCellValue());// 周三代课老师
                        }
                    } else if (c == 7) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setThursdayCurriculum(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周四科目
                        } else {
                            user.setThursdayCurriculum(cell.getStringCellValue());// 周四科目
                        }
                    } else if (c == 8) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setThursdayTeacher(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周四代课老师
                        } else {
                            user.setThursdayTeacher(cell.getStringCellValue());// 周四代课老师
                        }
                    } else if (c == 9) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setFridayCurriculum(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周五科目
                        } else {
                            user.setFridayCurriculum(cell.getStringCellValue());// 周五科目
                        }
                    } else if (c == 10) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setFridayTeacher(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 周五代课老师
                        } else {
                            user.setFridayTeacher(cell.getStringCellValue());// 周五代课老师
                        }
                    }
                }
            }
            // 添加到list
            userList.add(user);
        }
        return userList;
    }

    /**
     * 验证EXCEL文件
     * @param filePath
     * @return
     */
    public boolean validateExcel(String filePath) {
        if (filePath == null || !(isExcel2003(filePath) || isExcel2007(filePath))) {
            errorMsg = "文件名不是excel格式";
            return false;
        }
        return true;
    }

    // @描述：是否是2003的excel，返回true是2003
    public static boolean isExcel2003(String filePath) {
        return filePath.matches("^.+\\.(?i)(xls)$");
    }

    // @描述：是否是2007的excel，返回true是2007
    public static boolean isExcel2007(String filePath) {
        return filePath.matches("^.+\\.(?i)(xlsx)$");
    }
}
