package com.ipsa.learn.util;

import com.ipsa.learn.entity.CourseScore;
import com.ipsa.learn.entity.Curriculum;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Mir.Li
 * @date 2020/9/2  -  11:06
 */
public class ReadExcel01 {
    // 总行数
    private int totalRows = 0;
    // 总条数
    private int totalCells = 0;
    // 错误信息接收器
    private String errorMsg;

    // 构造方法
    public ReadExcel01() {
    }

    // 获取总行数
    public int getTotalRows() {
        return totalRows;
    }

    // 获取总列数
    public int getTotalCells() {
        return totalCells;
    }

    // 获取错误信息
    public String getErrorInfo() {
        return errorMsg;
    }

    /**
     * 读EXCEL文件，获取信息集合
     *
     * @param mFile
     * @return
     */
    public List<CourseScore> getExcelInfo(MultipartFile mFile) {
        String fileName = mFile.getOriginalFilename();// 获取文件名
        try {
            if (!validateExcel(fileName)) {// 验证文件名是否合格
                return null;
            }
            boolean isExcel2003 = true;// 根据文件名判断文件是2003版本还是2007版本
            if (isExcel2007(fileName)) {
                isExcel2003 = false;
            }
            List<CourseScore> userList = createExcel(mFile.getInputStream(), isExcel2003);
            return userList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
    public List<CourseScore> getExcelInfo2(MultipartFile mFile, String subName) {
        String fileName = mFile.getOriginalFilename();// 获取文件名
        try {
            if (!validateExcel(fileName)) {// 验证文件名是否合格
                return null;
            }
            boolean isExcel2003 = true;// 根据文件名判断文件是2003版本还是2007版本
            if (isExcel2007(fileName)) {
                isExcel2003 = false;
            }
            List<CourseScore> userList = createExcel2(mFile.getInputStream(), isExcel2003, subName);
            return userList;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据excel里面的内容读取客户信息
     *
     * @param is
     * @param isExcel2003 excel是2003还是2007版本
     * @return
     * @throws IOException
     */
    public List<CourseScore> createExcel(InputStream is, boolean isExcel2003) {
        try {
            Workbook wb = null;
            if (isExcel2003) {// 当excel是2003时,创建excel2003
                wb = new HSSFWorkbook(is);
            } else {// 当excel是2007时,创建excel2007
                wb = new XSSFWorkbook(is);
            }
            List<CourseScore> userList = readExcelValue(wb);// 读取Excel里面客户的信息
            return userList;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
    public List<CourseScore> createExcel2(InputStream is, boolean isExcel2003, String subName) {
        try {
            Workbook wb = null;
            if (isExcel2003) {// 当excel是2003时,创建excel2003
                wb = new HSSFWorkbook(is);
            } else {// 当excel是2007时,创建excel2007
                wb = new XSSFWorkbook(is);
            }
            List<CourseScore> userList = readExcelValue2(wb, subName);// 读取Excel里面客户的信息
            return userList;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 读取Excel里面客户的信息
     *
     * @param wb
     * @return
     */
    private List<CourseScore> readExcelValue(Workbook wb) {
        // 得到第一个shell
        Sheet sheet = wb.getSheetAt(0);
        // 得到Excel的行数
        this.totalRows = sheet.getPhysicalNumberOfRows();
        // 得到Excel的列数(前提是有行数)
        if (totalRows > 1 && sheet.getRow(0) != null) {
            //this.totalCells = sheet.getRow(1).getPhysicalNumberOfCells();
            this.totalCells = sheet.getRow(0).getLastCellNum();
            //this.totalCells = 4;
        }
        List<CourseScore> userList = new ArrayList<CourseScore>();
        // 循环Excel行数
        for (int r = 1; r < totalRows; r++) {
            Row row = sheet.getRow(r);
            if (row == null) {
                continue;
            }
            CourseScore user = new CourseScore();
            // 循环Excel的列
            for (int c = 0; c < this.totalCells; c++) {
                Cell cell = row.getCell(c);
                if (null != cell) {
                    if (c == 0) {
                        // 如果是纯数字,比如你写的是25,cell.getNumericCellValue()获得是25.0,通过截取字符串去掉.0获得25
                        if (cell.getCellType() == CellType.NUMERIC) {
                            String name = String.valueOf(cell.getNumericCellValue());
                            user.setStudentUserId(name.substring(0, name.length() - 2 > 0 ? name.length() - 2 : 1));// 学生Id
                        } else {
                            user.setStudentUserId(cell.getStringCellValue());// 学生Id
                        }
                    } else if (c == 1) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String sex = String.valueOf(cell.getNumericCellValue());
                            user.setStudentNo(sex.substring(0, sex.length() - 2 > 0 ? sex.length() - 2 : 1));// 学生编号
                        } else {
                            user.setStudentNo(cell.getStringCellValue());// 学生编号
                        }
                    } else if (c == 2) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setStudentUserName(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 学生名称
                        } else {
                            user.setStudentUserName(cell.getStringCellValue());// 学生名称
                        }
                    } else if (c == 3) {
                        if (cell.getCellType() ==  CellType.NUMERIC) {
                            String age = String.valueOf(cell.getNumericCellValue());
                            user.setSubjectsScores(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 科目分数
                        } else {
                            user.setSubjectsScores(cell.getStringCellValue());// 科目分数
                        }
                    }
                }
            }
            // 添加到list
            userList.add(user);
        }
        return userList;
    }
    private List<CourseScore> readExcelValue2(Workbook wb, String subName) {



        // 得到第一个shell
        Sheet sheet = wb.getSheetAt(0);

        StringBuilder builder = new StringBuilder();
        short lastCellNum = sheet.getRow(0).getLastCellNum();
        int scoreNum = lastCellNum - 3;
        for (int i = 0; i < scoreNum; i++) {
            builder.append(sheet.getRow(0).getCell(i + 3).getStringCellValue().substring(0,2)).append(",");
        }

        String subNames = builder.toString();
        String[] array = subNames.split(",");
        int indexVal = 0;
        for(int i = 0;i<array.length;i++){
            if(array[i].equals(subName)){
                indexVal = i + 3;
            }
        }

        // 得到Excel的行数
        this.totalRows = sheet.getPhysicalNumberOfRows();
        // 得到Excel的列数(前提是有行数)
        if (totalRows > 1 && sheet.getRow(0) != null) {
            //this.totalCells = sheet.getRow(1).getPhysicalNumberOfCells();
            this.totalCells = sheet.getRow(0).getLastCellNum();
            //this.totalCells = 4;
        }
        List<CourseScore> userList = new ArrayList<CourseScore>();
        // 循环Excel行数
        for (int r = 1; r < totalRows; r++) {
            Row row = sheet.getRow(r);
            if (row == null) {
                continue;
            }

           // for (int i = 0; i < this.totalCells - 3; i++) {
                CourseScore user = new CourseScore();
                // 循环Excel的列
                for (int c = 0; c < this.totalCells; c++) {
                    Cell cell = row.getCell(c);
                    if (null != cell) {
                        if (c == 0) {
                            // 如果是纯数字,比如你写的是25,cell.getNumericCellValue()获得是25.0,通过截取字符串去掉.0获得25
                            if (cell.getCellType() == CellType.NUMERIC) {
                                String name = String.valueOf(cell.getNumericCellValue());
                                user.setStudentUserId(name.substring(0, name.length() - 2 > 0 ? name.length() - 2 : 1));// 学生Id
                            } else {
                                user.setStudentUserId(cell.getStringCellValue());// 学生Id
                            }
                        } else if (c == 1) {
                            if (cell.getCellType() ==  CellType.NUMERIC) {
                                String sex = String.valueOf(cell.getNumericCellValue());
                                user.setStudentNo(sex.substring(0, sex.length() - 2 > 0 ? sex.length() - 2 : 1));// 学生编号
                            } else {
                                user.setStudentNo(cell.getStringCellValue());// 学生编号
                            }
                        } else if (c == 2) {
                            if (cell.getCellType() ==  CellType.NUMERIC) {
                                String age = String.valueOf(cell.getNumericCellValue());
                                user.setStudentUserName(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 学生名称
                            } else {
                                user.setStudentUserName(cell.getStringCellValue());// 学生名称
                            }
                        } else if (c == indexVal) {
                            if (cell.getCellType() ==  CellType.NUMERIC) {
                                String age = String.valueOf(cell.getNumericCellValue());
                                user.setSubjectsScores(age.substring(0, age.length() - 2 > 0 ? age.length() - 2 : 1));// 科目分数
                            } else {
                                user.setSubjectsScores(cell.getStringCellValue());// 科目分数
                            }
                        }
                    }
                }
                // 添加到list
                userList.add(user);
            //}
        }
        return userList;
    }

    /**
     * 验证EXCEL文件
     * @param filePath
     * @return
     */
    public boolean validateExcel(String filePath) {
        if (filePath == null || !(isExcel2003(filePath) || isExcel2007(filePath))) {
            errorMsg = "文件名不是excel格式";
            return false;
        }
        return true;
    }

    // @描述：是否是2003的excel，返回true是2003
    public static boolean isExcel2003(String filePath) {
        return filePath.matches("^.+\\.(?i)(xls)$");
    }

    // @描述：是否是2007的excel，返回true是2007
    public static boolean isExcel2007(String filePath) {
        return filePath.matches("^.+\\.(?i)(xlsx)$");
    }
}
