package com.ipsa.learn.util;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.security.SignatureException;
import java.util.*;

/** 
* 功能：MD5签名处理核心文件，不需要修改
* 版本：1.0
* 修改日期：2014-09-19
* 说明：
* */

public class SignCheck {
	
	private static Logger logger = LoggerFactory.getLogger(SignCheck.class);

    /**
     * 签名字符串
     * @param text 需要签名的字符串
     * @param key 密钥
     * @param input_charset 编码格式
     * @return 签名结果
     */
    public static String sign(String text, String key, String input_charset) {
    	text = text + key;
        return DigestUtils.md5Hex(getContentBytes(text, input_charset));
    }
    
    /**
     * 签名字符串
     * @param text 需要签名的字符串
     * @param sign 签名结果
     * @param key 密钥
     * @param input_charset 编码格式
     * @return 签名结果
     */
    public static boolean verify(String text, String sign, String key, String input_charset) {
    	text = text + key;
    	String mysign = DigestUtils.md5Hex(getContentBytes(text, input_charset));
    	logger.info("签名之后的sign：" + mysign);
    	if(mysign.equalsIgnoreCase(sign)) {
    		return true;
    	}
    	else {
    		return false;
    	}
    	
    }

    /**
     * @param content
     * @param charset
     * @return
     * @throws SignatureException
     * @throws UnsupportedEncodingException 
     */
    private static byte[] getContentBytes(String content, String charset) {
        if (charset == null || "".equals(charset)) {
            return content.getBytes();
        }
        try {
            return content.getBytes(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("MD5签名过程中出现错误,指定的编码集不对,您目前指定的编码集是:" + charset);
        }
    }

    /** 
     * 除去数组中的空值和签名参数
     * @param sArray 签名参数组
     * @return 去掉空值与签名参数后的新签名参数组
     */
    public static Map<String, String> paraFilter(Map<String, String> sArray) {

        Map<String, String> result = new HashMap<String, String>();

        if (sArray == null || sArray.size() <= 0) {
            return result;
        }

        for (String key : sArray.keySet()) {
            String value = sArray.get(key);
            if (value == null || value.equals("") || key.equalsIgnoreCase("sign")
                || key.equalsIgnoreCase("sign_type")) {
                continue;
            }
            result.put(key, value);
        }
        
        return result;
    }

    /** 
     * 把数组所有元素排序，并按照“参数=参数值”的模式用“&”字符拼接成字符串
     * @param params 需要排序并参与字符拼接的参数组
     * @return 拼接后字符串
     */
    public static String createLinkString(Map<String, String> params) {

        List<String> keys = new ArrayList<String>(params.keySet());
        Collections.sort(keys);

        String prestr = "";

        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key);

            if (i == keys.size() - 1) {//拼接时，不包括最后一个&字符
                prestr = prestr + key + "=" + value;
            } else {
                prestr = prestr + key + "=" + value + "&";
            }
        }

        return prestr;
    }
    
	public static String sendPostInfo(Map<String, String> sParaTemp,
			String gateway, String key) throws Exception {
		// 待请求参数数组
		Map<String, String> sPara = buildRequestPara(sParaTemp, key);

		HttpProtocolHandler httpProtocolHandler = HttpProtocolHandler
				.getInstance();

		HttpRequest request = new HttpRequest(HttpResultType.BYTES);
		// 设置编码集
		request.setCharset(AlipayConfig.input_charset);

		request.setParameters(generatNameValuePair(sPara));
		request.setUrl(gateway);

		HttpResponse response = httpProtocolHandler.execute(request);
		if (response == null) {
			return null;
		}

		String strResult = response.getStringResult();

		return strResult;
	}
	
	public static Map<String, String> buildRequestPara(
			Map<String, String> sParaTemp, String key) {
		// 除去数组中的空值和签名参数
		Map<String, String> sPara = AlipayCore.paraFilter(sParaTemp);
		// 生成签名字符串
		String signString = SignCheck.createLinkString(sPara);
		String input_charset = sPara.get("_input_charset");
		if(StringUtils.isBlank(input_charset)){
			input_charset = "utf-8";
		}
		String mysign = SignCheck.sign(signString, key, input_charset);
		// 签名结果与签名方式加入请求提交参数组中
		sPara.put("sign", mysign);
//		sPara.put("sign_type", "MD5");

		return sPara;
	}
	
	/**
	 * MAP类型数组转换成NameValuePair类型
	 * 
	 * @param properties
	 *            MAP类型数组
	 * @return NameValuePair类型数组
	 */
	private static NameValuePair[] generatNameValuePair(
			Map<String, String> properties) {
		NameValuePair[] nameValuePair = new NameValuePair[properties.size()];
		int i = 0;
		for (Map.Entry<String, String> entry : properties.entrySet()) {
			nameValuePair[i++] = new NameValuePair(entry.getKey(),
					entry.getValue());
		}

		return nameValuePair;
	}
	
	/* 测试异步调用*/
	/*
	public static void main(String[] args){
		Map<String, String> sPara = new HashMap<String, String>();
		sPara.put("is_success", "true");
		sPara.put("channel_id", "WANDA_WAP");
		sPara.put("total_fee", "0.01");
		sPara.put("out_trade_no", "f2ae68b8-389d-4730-a671-2b7d0d02816c");
		sPara.put("_input_charset", "utf-8");
		//支付的交易号
		sPara.put("trade_no", "2014101579949324");
		sPara.put("pay_method", "ALIPAY");
		sPara.put("pay_status", "TRADE_PAID");
		
		try {
			String resp = sendPostInfo(sPara, "http://10.199.202.58/waporder/orderPayNotify.html", "asdfghjklqw0gghhhrf6");
			System.out.println("resp:" + resp);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
}