package com.ipsa.learn.util;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.util.*;

/**
 * xml工具类
 * @author miklchen
 *
 */
public class XMLUtil {

	/**
	 * 解析xml,返回第一级元素键值对。如果第一级元素有子节点，则此节点的值是子节点的xml数据。
	 * @param strxml
	 * @return
	 * @throws JDOMException
	 * @throws IOException
	 */
	public static Map doXMLParse(String strxml) throws JDOMException, IOException {
		if(null == strxml || "".equals(strxml)) {
			return null;
		}
		
		Map m = new HashMap();
		InputStream in = HttpClientUtil.String2Inputstream(strxml);
		SAXBuilder builder = new SAXBuilder();
		
		//解决XML外部实体注入漏洞，2018-7-19 14:43:38 ls添加
		String FEATURE = "http://apache.org/xml/features/disallow-doctype-decl";
		builder.setFeature(FEATURE, true);
		FEATURE = "http://xml.org/sax/features/external-general-entities";
		builder.setFeature(FEATURE, false);
		FEATURE = "http://xml.org/sax/features/external-parameter-entities";
		builder.setFeature(FEATURE, false);
		FEATURE = "http://apache.org/xml/features/nonvalidating/load-external-dtd";
		builder.setFeature(FEATURE, false);
		
		Document doc = builder.build(in);
		Element root = doc.getRootElement();
		List list = root.getChildren();
		Iterator it = list.iterator();
		while(it.hasNext()) {
			Element e = (Element) it.next();
			String k = e.getName();
			String v = "";
			List children = e.getChildren();
			if(children.isEmpty()) {
				v = e.getTextNormalize();
			} else {
				v = XMLUtil.getChildrenText(children);
			}
			
			m.put(k, v);
		}
		
		//关闭流
		in.close();
		
		return m;
	}
	
	/**
	 * 获取子结点的xml
	 * @param children
	 * @return String
	 */
	public static String getChildrenText(List children) {
		StringBuffer sb = new StringBuffer();
		if(!children.isEmpty()) {
			Iterator it = children.iterator();
			while(it.hasNext()) {
				Element e = (Element) it.next();
				String name = e.getName();
				String value = e.getTextNormalize();
				List list = e.getChildren();
				sb.append("<" + name + ">");
				if(!list.isEmpty()) {
					sb.append(XMLUtil.getChildrenText(list));
				}
				sb.append(value);
				sb.append("</" + name + ">");
			}
		}
		
		return sb.toString();
	}
	
	/**
	 * 获取xml编码字符集
	 * @param strxml
	 * @return
	 * @throws IOException 
	 * @throws JDOMException 
	 */
	public static String getXMLEncoding(String strxml) throws JDOMException, IOException {
		InputStream in = HttpClientUtil.String2Inputstream(strxml);
		SAXBuilder builder = new SAXBuilder();
		Document doc = builder.build(in);
		in.close();
		return (String)doc.getProperty("encoding");
	}
	
	public static String parseToXML(Map<String, String> dataMap) {
		StringBuilder strBuilder = new StringBuilder();
		strBuilder.append("<xml>");
		Set<String> objSet = dataMap.keySet();
		for (Object key : objSet) {
			if (key == null) {
				continue;
			}
			strBuilder.append("<").append(key.toString()).append(">");
			Object value = dataMap.get(key);
			strBuilder.append(coverter(value));
			strBuilder.append("</").append(key.toString()).append(">");
		}
		strBuilder.append("</xml>");
		return strBuilder.toString();
	}
	
	/**
	 * Coverter.
	 * 
	 * @param object
	 *            the object
	 * @return the string
	 */
	public static String coverter(Object object) {
		if (object instanceof Object[]) {
			return coverter((Object[]) object);
		}
		if (object instanceof Collection) {
			return coverter((Collection<?>) object);
		}
		StringBuilder strBuilder = new StringBuilder();
		if (isObject(object)) {
			Class<? extends Object> clz = object.getClass();
			Field[] fields = clz.getDeclaredFields();

			for (Field field : fields) {
				field.setAccessible(true);
				if (field == null) {
					continue;
				}
				String fieldName = field.getName();
				Object value = null;
				try {
					value = field.get(object);
				} catch (IllegalArgumentException e) {
					continue;
				} catch (IllegalAccessException e) {
					continue;
				}
				strBuilder.append("<").append(fieldName)
						.append(" className=\"")
						.append(value.getClass().getName()).append("\">");
				if (isObject(value)) {
					strBuilder.append(coverter(value));
				} else if (value == null) {
					strBuilder.append("null");
				} else {
					strBuilder.append(value.toString());
				}
				strBuilder.append("</").append(fieldName).append(">");
			}
		} else if (object == null) {
			strBuilder.append("null");
		} else {
			strBuilder.append(object.toString());
		}
		return strBuilder.toString();
	}

	/**
	 * Checks if is object.
	 * 
	 * @param obj
	 *            the obj
	 * 
	 * @return true, if is object
	 */
	private static boolean isObject(Object obj) {
		if (obj == null) {
			return false;
		}
		if (obj instanceof String) {
			return false;
		}
		if (obj instanceof Integer) {
			return false;
		}
		if (obj instanceof Double) {
			return false;
		}
		if (obj instanceof Float) {
			return false;
		}
		if (obj instanceof Byte) {
			return false;
		}
		if (obj instanceof Long) {
			return false;
		}
		if (obj instanceof Character) {
			return false;
		}
		if (obj instanceof Short) {
			return false;
		}
		if (obj instanceof Boolean) {
			return false;
		}
		return true;
	}
	
	public static void main(String[] args){
		String noncestr = Sha1Util.getNonceStr();
		String timestamp = Sha1Util.getTimeStamp();
    
		Map<String, String> signParams = new TreeMap<String, String>();
		signParams.put("appid", "1232434");
		signParams.put("mch_id", "887623425213123");
		signParams.put("nonce_str", noncestr);
		signParams.put("timestamp", timestamp);
		signParams.put("body", "产品");
		signParams.put("out_trade_no", "2015072915303345");
//		String isDev = ApplicationInitService.getPropertiesValue("isDev");
//		String total_fee = orderForm.getOrderAmount().multiply(new BigDecimal(100)).toString();
		String total_fee = "1";
//		if(Boolean.parseBoolean(isDev)){
//			total_fee = "1";
//		}
		signParams.put("total_fee", total_fee);
		signParams.put("trade_type", "NATIVE");
		signParams.put("product_id", "product_id");
		signParams.put("spbill_create_ip", "127.0.0.1");
		signParams.put("notify_url", "http://localhost:8080/pay" + "/wechat/notify");
		
		String element = new XMLUtil().parseToXML(signParams);
		System.out.println(element);
		
	}
}
